@@ -1,16 +1,16 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: buildhash.c,v 1.64 1995/01/08 23:23:26 geoff Exp $";
+    "$Id: buildhash.c,v 1.67 2000/08/22 10:52:25 geoff Exp $";
 #endif
 
 #define MAIN
 
 /*
- * buildhash.c - make a hash table for okspell
+ * buildhash.c - make a hash table for ispell
  *
  * Pace Willisson, 1983
  *
- * Copyright 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * Copyright 1992, 1993, 1999, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,6 +49,16 @@ static char Rcs_Id[] =
 
 /*
  * $Log: buildhash.c,v $
+ * Revision 1.67  2000/08/22 10:52:25  geoff
+ * Fix a whole bunch of signed/unsigned compiler warnings.
+ *
+ * Revision 1.66  1999/01/07 01:22:32  geoff
+ * Update the copyright.
+ *
+ * Revision 1.65  1997/12/02  06:24:33  geoff
+ * Get rid of an obsolete reference to "okspell".  Get rid of some
+ * compile options that really shouldn't be optional.
+ *
  * Revision 1.64  1995/01/08  23:23:26  geoff
  * Make the various file suffixes configurable for DOS purposes.
  *
@@ -106,7 +116,7 @@ int main (argc, argv)
     FILE *	countf;
     FILE *	statf;
     int		stats[NSTAT];
-    int		i;
+    unsigned int i;
     int		j;
 
     while (argc > 1  &&  *argv[1] == '-')
@@ -213,7 +223,7 @@ static void output ()
     register struct dent *	dp;
     int				strptr;
     int				n;
-    int				i;
+    unsigned int		i;
     int				maxplen;
     int				maxslen;
     struct flagent *		fentry;
@@ -235,7 +245,7 @@ static void output ()
     ** without damage.
     */
     maxslen = 0;
-    for (i = numsflags, fentry = sflaglist;  --i >= 0;  fentry++)
+    for (i = 0, fentry = sflaglist;  i < numsflags;  i++, fentry++)
 	{
 	if (fentry->stripl)
 	    {
@@ -258,7 +268,7 @@ static void output ()
 	    maxslen = n;
 	}
     maxplen = 0;
-    for (i = numpflags, fentry = pflaglist;  --i >= 0;  fentry++)
+    for (i = 0, fentry = pflaglist;  i < numpflags;  i++, fentry++)
 	{
 	if (fentry->stripl)
 	    {
@@ -286,8 +296,8 @@ static void output ()
     hashheader.strtypestart = strptr;
     for (i = 0;  i < hashheader.nstrchartype;  i++)
 	{
-	n = strlen (chartypes[i].name) + 1;
-	(void) fwrite (chartypes[i].name, n, 1, houtfile);
+	n = strlen ((char *) chartypes[i].name) + 1;
+	(void) fwrite ((char *) chartypes[i].name, n, 1, houtfile);
 	strptr += n;
 	n = strlen (chartypes[i].deformatter) + 1;
 	(void) fwrite (chartypes[i].deformatter, n, 1, houtfile);
@@ -313,12 +323,12 @@ static void output ()
     for (i = 0, dp = hashtbl;  i < hashsize;  i++, dp++)
 	{
 	if (dp->word == NULL)
-	    dp->word = (char *) -1;
+	    dp->word = (unsigned char *) -1;
 	else
 	    {
-	    n = strlen (dp->word) + 1;
+	    n = strlen ((char *) dp->word) + 1;
 	    (void) fwrite (dp->word, n, 1, houtfile);
-	    dp->word = (char *) strptr;
+	    dp->word = (unsigned char *) strptr;
 	    strptr += n;
 	    }
 	}
@@ -439,7 +449,7 @@ VOID * mymalloc (size)		/* Fast, unfree-able variant of malloc */
     unsigned int	size;
     {
     VOID *		retval;
-    static int		bytesleft = 0;
+    static unsigned int	bytesleft = 0;
     static VOID *	nextspace;
 
     if (size < 4)
@@ -487,8 +497,8 @@ static void readdict ()
     struct dent		d;
     register struct dent * dp;
     struct dent *	lastdp;
-    char		lbuf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
-    char		ucbuf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
+    unsigned char	lbuf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
+    unsigned char	ucbuf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
     FILE *		dictf;
     int			i;
     int			h;
@@ -508,7 +518,7 @@ static void readdict ()
 	}
 
     i = 0;
-    while (fgets (lbuf, sizeof lbuf, dictf) != NULL)
+    while (fgets ((char *) lbuf, sizeof lbuf, dictf) != NULL)
 	{
 	if (!silent  &&  (i % 1000) == 0)
 	    {
@@ -526,7 +536,6 @@ static void readdict ()
 	if ((dp->flagfield & USED) == 0)
 	    {
 	    *dp = d;
-#ifndef NO_CAPITALIZATION_SUPPORT
 	    /*
 	    ** If it's a followcase word, we need to make this a
 	    ** special dummy entry, and add a second with the
@@ -537,7 +546,6 @@ static void readdict ()
 		if (addvheader (dp))
 		  exit (1);
 		}
-#endif
 	    }
 	else
 	    {
@@ -548,16 +556,14 @@ static void readdict ()
 	    ** word.  Note that d.word always exists at
 	    ** this point.
 	    */
-	    (void) strcpy (ucbuf, d.word);
+	    (void) strcpy ((char *) ucbuf, (char *) d.word);
 	    chupcase (ucbuf);
 	    while (dp != NULL)
 		{
-		if (strcmp (dp->word, ucbuf) == 0)
+		if (strcmp ((char *) dp->word, (char *) ucbuf) == 0)
 		    break;
-#ifndef NO_CAPITALIZATION_SUPPORT
 		while (dp->flagfield & MOREVARIANTS)
 		    dp = dp->next;
-#endif /* NO_CAPITALIZATION_SUPPORT */
 		dp = dp->next;
 		}
 	    if (dp != NULL)
@@ -584,7 +590,6 @@ static void readdict ()
 		*dp = d;
 		lastdp->next = dp;
 		dp->next = NULL;
-#ifndef NO_CAPITALIZATION_SUPPORT
 		/*
 		** If it's a followcase word, we need to make this a
 		** special dummy entry, and add a second with the
@@ -595,7 +600,6 @@ static void readdict ()
 		    if (addvheader (dp))
 		      exit (1);
 		    }
-#endif
 		}
 	    }
 	}
@@ -606,18 +610,14 @@ static void readdict ()
 
 static void newcount ()
     {
-    char		buf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
-#ifndef NO_CAPITALIZATION_SUPPORT
+    unsigned char	buf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
     ichar_t		ibuf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
-#endif
     register FILE *	d;
     register int	i;
-#ifndef NO_CAPITALIZATION_SUPPORT
     ichar_t		lastibuf[sizeof ibuf / sizeof (ichar_t)];
     int			headercounted;
     int			followcase;
     register char *	cp;
-#endif
 
     if (!silent)
 	(void) fprintf (stderr, BHASH_C_COUNTING);
@@ -628,19 +628,16 @@ static void newcount ()
 	exit (1);
 	}
 
-#ifndef NO_CAPITALIZATION_SUPPORT
     headercounted = 0;
     lastibuf[0] = 0;
-#endif
-    for (i = 0;  fgets (buf, sizeof buf, d);  )
+    for (i = 0;  fgets ((char *) buf, sizeof buf, d);  )
 	{
 	if ((++i % 1000) == 0  &&  !silent)
 	    {
 	    (void) fprintf (stderr, "%d ", i);
 	    (void) fflush (stdout);
 	    }
-#ifndef NO_CAPITALIZATION_SUPPORT
-	cp = index (buf, hashheader.flagmarker);
+	cp = index ((char *) buf, hashheader.flagmarker);
 	if (cp != NULL)
 	    *cp = '\0';
 	if (strtoichar (ibuf, buf, INPUTWORDLEN * sizeof (ichar_t), 1))
@@ -670,7 +667,6 @@ static void newcount ()
 	    headercounted = 1;
 	    }
 	(void) icharcpy (lastibuf, ibuf);
-#endif
 	}
     (void) fclose (d);
     if (!silent)
