@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: correct.c,v 1.58 1994/11/02 06:56:00 geoff Exp $";
+    "$Id: correct.c,v 1.74 2001/06/10 23:54:56 geoff Exp $";
 #endif
 
 /*
@@ -11,7 +11,7 @@ static char Rcs_Id[] =
  *
  * Copyright (c), 1983, by Pace Willisson
  *
- * Copyright 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * Copyright 1992, 1993, 1999, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,6 +50,63 @@ static char Rcs_Id[] =
 
 /*
  * $Log: correct.c,v $
+ * Revision 1.74  2001/06/10 23:54:56  geoff
+ * Fix an ask-mode bug that could cause hangs.
+ *
+ * Revision 1.73  2001/06/07 08:02:18  geoff
+ * When copying out, be sure to use the information from contextbufs[0]
+ * (the original file) rather then filteredbuf (the deformatted
+ * information).
+ *
+ * Revision 1.72  2000/08/24 06:47:16  geoff
+ * Fix a dumb error in merging the correct_verbose_mode patch
+ *
+ * Revision 1.71  2000/08/22 10:52:25  geoff
+ * Fix a whole bunch of signed/unsigned compiler warnings.
+ *
+ * Revision 1.70  2000/08/22 00:11:25  geoff
+ * Add support for correct_verbose_mode.
+ *
+ * Revision 1.69  1999/11/04 07:51:05  geoff
+ * In verbose ask mode, put out a newline after the EOF so the TTY looks
+ * nice.
+ *
+ * Revision 1.68  1999/11/04 07:19:59  geoff
+ * Add "oktochange" to inserttoken, and set it to false on the first of
+ * each paired call.
+ *
+ * Revision 1.67  1999/10/05 05:49:56  geoff
+ * When processing the ~ command, don't pass a newline by accident!
+ *
+ * Revision 1.66  1999/01/18 03:28:24  geoff
+ * Turn many char declarations into unsigned char, so that we won't have
+ * sign-extension problems.
+ *
+ * Revision 1.65  1999/01/07  01:57:51  geoff
+ * Update the copyright.
+ *
+ * Revision 1.64  1999/01/03  01:46:25  geoff
+ * Add support for external deformatters.  Also display tab characters
+ * correctly when showing context.
+ *
+ * Revision 1.63  1997/12/02  06:24:38  geoff
+ * Get rid of some compile options that really shouldn't be optional.
+ *
+ * Revision 1.62  1997/12/01  00:53:43  geoff
+ * Don't strip out the newline at the end of contextbufs on long lines.
+ *
+ * Revision 1.61  1995/11/08  05:09:26  geoff
+ * Add the new interactive mode ("askverbose").  Modify the deformatting
+ * support to allow html/sgml as a full equal of nroff and TeX.
+ *
+ * Revision 1.60  1995/10/25  04:05:28  geoff
+ * Line added by Gerry Tierney to reset insidehtml flag for each new file
+ * in case a tag was left open by a previous file.  10/14/95.
+ *
+ * Revision 1.59  1995/08/05  23:19:43  geoff
+ * Fix a bug that caused offsets for long lines to be confused if the
+ * line started with a quoting uparrow.
+ *
  * Revision 1.58  1994/11/02  06:56:00  geoff
  * Remove the anyword feature, which I've decided is a bad idea.
  *
@@ -123,20 +180,21 @@ static char Rcs_Id[] =
 
 void		givehelp P ((int interactive));
 void		checkfile P ((void));
-void		correct P ((char * ctok, int ctokl, ichar_t * itok, int itokl,
-		  char ** curchar));
-static void	show_line P ((char * line, char * invstart, int invlen));
-static int	show_char P ((char ** cp, int linew, int output, int maxw));
-static int	line_size P ((char * buf, char * bufend));
-static void	inserttoken P ((char * buf, char * start, char * tok,
-		  char ** curchar));
-static int	posscmp P ((char * a, char * b));
-int		casecmp P ((char * a, char * b, int canonical));
+void		correct P ((unsigned char * ctok, int ctokl, ichar_t * itok,
+		  int itokl, unsigned char ** curchar));
+static void	show_line P ((unsigned char * line, unsigned char * invstart,
+		  int invlen));
+static int	show_char P ((unsigned char ** cp, int linew, int output,
+		  int maxw));
+static int	line_size P ((unsigned char * buf, unsigned char * bufend));
+static void	inserttoken P ((unsigned char * buf, unsigned char * start,
+		  unsigned char * tok, unsigned char ** curchar,
+		  int oktochange));
+static int	posscmp P ((unsigned char * a, unsigned char * b));
+int		casecmp P ((unsigned char * a, unsigned char * b, int canonical));
 void		makepossibilities P ((ichar_t * word));
 static int	insert P ((ichar_t * word));
-#ifndef NO_CAPITALIZATION_SUPPORT
 static void	wrongcapital P ((ichar_t * word));
-#endif /* NO_CAPITALIZATION_SUPPORT */
 static void	wrongletter P ((ichar_t * word));
 static void	extraletter P ((ichar_t * word));
 static void	missingletter P ((ichar_t * word));
@@ -159,8 +217,8 @@ static void	save_root_cap P ((ichar_t * word, ichar_t * pattern,
 		  int * nsaved));
 static char *	getline P ((char * buf));
 void		askmode P ((void));
-void		copyout P ((char ** cc, int cnt));
-static void	lookharder P ((char * string));
+void		copyout P ((unsigned char ** cc, int cnt));
+static void	lookharder P ((unsigned char * string));
 #ifdef REGEX_LOOKUP
 static void	regex_dict_lookup P ((char * cmd, char * grepstr));
 #endif /* REGEX_LOOKUP */
@@ -226,31 +284,45 @@ void givehelp (interactive)
 void checkfile ()
     {
     int		bufno;
-    int		bufsize;
+    unsigned int bufsize;
     int		ch;
 
+    insidehtml = 0;
+    math_mode = 0;
+    LaTeX_Mode = 'P';
+
     for (bufno = 0;  bufno < contextsize;  bufno++)
 	contextbufs[bufno][0] = '\0';
 
     for (  ;  ;  )
 	{
 	for (bufno = contextsize;  --bufno > 0;  )
-	    (void) strcpy (contextbufs[bufno],
-	      contextbufs[bufno - 1]);
+	    (void) strcpy ((char *) contextbufs[bufno],
+	      (char *) contextbufs[bufno - 1]);
 	if (quit)	/* quit can't be set in l mode */
 	    {
-	    while (fgets (contextbufs[0],
-	      sizeof contextbufs[0], infile) != NULL)
-		(void) fputs (contextbufs[0], outfile);
+	    if (sourcefile == NULL)
+		sourcefile = infile;
+	    while (fgets ((char *) contextbufs[0], sizeof contextbufs[0],
+		sourcefile)
+	      != NULL)
+		(void) fputs ((char *) contextbufs[0], outfile);
 	    break;
 	    }
 	/*
 	 * Only read in enough characters to fill half this buffer so that any
 	 * corrections we make are not likely to cause an overflow.
 	 */
-	if (fgets (contextbufs[0], (sizeof contextbufs[0]) / 2, infile)
+	if (fgets ((char *) filteredbuf, sizeof filteredbuf / 2, infile)
 	  == NULL)
-	    break;
+	    {
+	    if (sourcefile == NULL)
+		break;
+	    while (fgets ((char *) contextbufs[0], sizeof contextbufs[0],
+		sourcefile)
+	      != NULL)
+		(void) fputs ((char *) contextbufs[0], outfile);
+	    }
 	/*
 	 * If we didn't read to end-of-line, we may have ended the
 	 * buffer in the middle of a word.  So keep reading until we
@@ -258,40 +330,59 @@ void checkfile ()
 	 * word. (or until the buffer is full, which fortunately isn't
 	 * all that likely).
 	 */
-	bufsize = strlen (contextbufs[0]);
-	if (bufsize == (sizeof contextbufs[0]) / 2 - 1)
+	bufsize = strlen ((char *) filteredbuf);
+	if (bufsize == sizeof filteredbuf / 2 - 1)
 	    {
-	    ch = (unsigned char) contextbufs[0][bufsize - 1];
-	    while (bufsize < sizeof contextbufs[0] - 1
+	    ch = (unsigned char) filteredbuf[bufsize - 1];
+	    while (bufsize < sizeof filteredbuf - 1
 	      &&  (iswordch ((ichar_t) ch)  ||  isboundarych ((ichar_t) ch)
 	      ||  isstringstart (ch)))
 		{
 		ch = getc (infile);
 		if (ch == EOF)
 		    break;
-		contextbufs[0][bufsize++] = (char) ch;
-		contextbufs[0][bufsize] = '\0';
+		filteredbuf[bufsize++] = (char) ch;
+		filteredbuf[bufsize] = '\0';
 		}
 	    }
+	/*
+	 * If we're not filtering, make a duplicate of filteredbuf in
+	 * contextbufs[0].  Otherwise, read the same number of
+	 * characters into contextbufs[0] from sourcefile.
+	 */
+	if (sourcefile == NULL)
+	    (void) strcpy ((char *) contextbufs[0], (char *) filteredbuf);
+	else
+	    {
+	    if (fread (contextbufs[0], 1, bufsize, sourcefile) != bufsize)
+		{
+		(void) fprintf (stderr, CORR_C_SHORT_SOURCE);
+		xflag = 0;		/* Preserve file backup */
+		break;
+		}
+	    contextbufs[0][bufsize] = '\0';
+	    }
 	checkline (outfile);
 	}
     }
 
 void correct (ctok, ctokl, itok, itokl, curchar)
-    char *		ctok;
+    unsigned char *	ctok;
     int			ctokl;
     ichar_t *		itok;
     int			itokl;
-    char **		curchar;
+    unsigned char **	curchar;    /* Pointer into filteredbuf */
     {
     register int	c;
     register int	i;
     int			col_ht;
+    unsigned char *	curcontextchar; /* Pointer into contextbufs[0] */
     int			ncols;
-    char *		start_l2;
-    char *		begintoken;
+    unsigned char *	start_l2;
+    unsigned char *	begintoken;
 
-    begintoken = *curchar - strlen (ctok);
+    curcontextchar = contextbufs[0] + (*curchar - filteredbuf);
+    begintoken = curcontextchar - strlen ((char *) ctok);
 
     if (icharlen (itok) <= minword)
 	return;			/* Accept very short words */
@@ -301,7 +392,7 @@ checkagain:
 	return;
 
     erase ();
-    (void) printf ("    %s", ctok);
+    (void) printf ("    %s", (char *) ctok);
     if (currentfile)
 	(void) printf (CORR_C_FILE_LABEL, currentfile);
     if (readonly)
@@ -355,12 +446,12 @@ checkagain:
 	show_line (contextbufs[i], contextbufs[i], 0);
 
     start_l2 = contextbufs[0];
-    if (line_size (contextbufs[0], *curchar) > co - (sg << 1) - 1)
+    if (line_size (contextbufs[0], curcontextchar) > co - (sg << 1) - 1)
 	{
 	start_l2 = begintoken - (co / 2);
 	while (start_l2 < begintoken)
 	    {
-	    i = line_size (start_l2, *curchar) + 1;
+	    i = line_size (start_l2, curcontextchar) + 1;
 	    if (i + (sg << 1) <= co)
 		break;
 	    start_l2 += i - co;
@@ -370,7 +461,7 @@ checkagain:
 	if (start_l2 < contextbufs[0])
 	    start_l2 = contextbufs[0];
 	}
-    show_line (start_l2, begintoken, (int) strlen (ctok));
+    show_line (start_l2, begintoken, (int) strlen ((char *) ctok));
 
     if (minimenusize != 0)
 	{
@@ -381,7 +472,7 @@ checkagain:
     for (  ;  ;  )
 	{
 	(void) fflush (stdout);
-	switch (c = (GETKEYSTROKE () & NOPARITY))
+	switch (c = GETKEYSTROKE ())
 	    {
 	    case 'Z' & 037:
 		stop ();
@@ -396,7 +487,7 @@ checkagain:
 		    {
 		    (void) printf (CORR_C_CONFIRM_QUIT);
 		    (void) fflush (stdout);
-		    c = (GETKEYSTROKE () & NOPARITY);
+		    c = GETKEYSTROKE ();
 		    }
 		else
 		    c = 'y';
@@ -435,11 +526,11 @@ checkagain:
 		goto checkagain;
 	    case '!':
 		{
-		char	buf[200];
+		unsigned char	buf[200];
 
 		move (li - 1, 0);
 		(void) putchar ('!');
-		if (getline (buf) == NULL)
+		if (getline ((char *) buf) == NULL)
 		    {
 		    (void) putchar (7);
 		    erase ();
@@ -449,9 +540,9 @@ checkagain:
 		(void) printf ("\r\n");
 		(void) fflush (stdout);
 #ifdef	USESH
-		shescape (buf);
+		shescape ((char *) buf);
 #else
-		(void) shellescape (buf);
+		(void) shellescape ((char *) buf);
 #endif
 		erase ();
 		goto checkagain;
@@ -464,7 +555,7 @@ checkagain:
 		    (void) printf ("%s ", CORR_C_READONLY);
 		    }
 		(void) printf (CORR_C_REPLACE_WITH);
-		if (getline (ctok) == NULL)
+		if (getline ((char *) ctok) == NULL)
 		    {
 		    (void) putchar (7);
 		    /* Put it back */
@@ -473,11 +564,14 @@ checkagain:
 		else
 		    {
 		    inserttoken (contextbufs[0],
-		      begintoken, ctok, curchar);
+		      begintoken, ctok, &curcontextchar, 0);
+		    inserttoken (filteredbuf,
+		      filteredbuf + (begintoken - contextbufs[0]),
+		      ctok, curchar, 1);
 		    if (strtoichar (itok, ctok, itokl, 0))
 			{
 			(void) putchar (7);
-			(void) printf (WORD_TOO_LONG (ctok));
+			(void) printf (WORD_TOO_LONG ((char *) ctok));
 			}
 		    changes = 1;
 		    }
@@ -490,7 +584,7 @@ checkagain:
 		i = c - '0';
 		if (easypossibilities >= 10)
 		    {
-		    c = GETKEYSTROKE () & NOPARITY;
+		    c = GETKEYSTROKE ();
 		    if (c >= '0'  &&  c <= '9')
 			i = i * 10 + c - '0';
 		    else if (c != '\r'  &&  c != '\n')
@@ -501,10 +595,13 @@ checkagain:
 		    }
 		if (i < easypossibilities)
 		    {
-		    (void) strcpy (ctok, possibilities[i]);
+		    (void) strcpy ((char *) ctok, possibilities[i]);
 		    changes = 1;
 		    inserttoken (contextbufs[0],
-			begintoken, ctok, curchar);
+			begintoken, ctok, &curcontextchar, 0);
+		    inserttoken (filteredbuf,
+		      filteredbuf + (begintoken - contextbufs[0]),
+		      ctok, curchar, 1);
 		    erase ();
 		    if (readonly)
 			{
@@ -523,10 +620,10 @@ checkagain:
 		break;
 	    case 'l': case 'L':
 		{
-		char	buf[100];
+		unsigned char	buf[100];
 		move (li - 1, 0);
 		(void) printf (CORR_C_LOOKUP_PROMPT);
-		if (getline (buf) == NULL)
+		if (getline ((char *) buf) == NULL)
 		    {
 		    (void) putchar (7);
 		    erase ();
@@ -551,30 +648,34 @@ checkagain:
     }
 
 static void show_line (line, invstart, invlen)
-    char *		line;
-    register char *	invstart;
+    unsigned char *	line;
+    register unsigned char *
+			invstart;
     register int	invlen;
     {
-    register int	width;
+    register int	width = 0;
+    int			maxwidth = co - 1;
 
-    width = invlen ? (sg << 1) : 0;
-    while (line < invstart  &&  width < co - 1)
+    if (invlen != 0)
+	maxwidth -= sg << 1;
+    while (line < invstart  &&  width < maxwidth)
 	width += show_char (&line, width, 1, invstart - line);
     if (invlen)
 	{
 	inverse ();
 	invstart += invlen;
-	while (line < invstart  &&  width < co - 1)
+	while (line < invstart  &&  width < maxwidth)
 	    width += show_char (&line, width, 1, invstart - line);
 	normal ();
 	}
-    while (*line  &&  width < co - 1)
+    while (*line  &&  width < maxwidth)
 	width += show_char (&line, width, 1, 0);
     (void) printf ("\r\n");
     }
 
 static int show_char (cp, linew, output, maxw)
-    register char **	cp;
+    register unsigned char **
+			cp;
     int			linew;
     int			output;		/* NZ to actually do output */
     int			maxw;		/* NZ to limit width shown */
@@ -585,7 +686,7 @@ static int show_char (cp, linew, output, maxw)
     ichar_t		ichar;
     register int	width;
 
-    ch = (unsigned char) **cp;
+    ch = **cp;
     if (l1_isstringch (*cp, len, 0))
 	ichar = SET_SIZE + laststringch;
     else
@@ -600,7 +701,10 @@ static int show_char (cp, linew, output, maxw)
     if (ch == '\t')
 	{
 	if (output)
-	    (void) putchar ('\t');
+	    {
+	    for (i = 8 - (linew & 0x07);  --i >= 0;  )
+		(void) putchar (' ');
+	    }
 	(*cp)++;
 	return 8 - (linew & 0x07);
 	}
@@ -648,8 +752,9 @@ static int show_char (cp, linew, output, maxw)
     }
 
 static int line_size (buf, bufend)
-    char *		buf;
-    register char *	bufend;
+    unsigned char *	buf;
+    register unsigned char *
+			bufend;
     {
     register int	width;
 
@@ -658,18 +763,22 @@ static int line_size (buf, bufend)
     return width;
     }
 
-static void inserttoken (buf, start, tok, curchar)
-    char *		buf;
-    char *		start; 
-    register char *	tok;
-    char **		curchar;
+static void inserttoken (buf, start, tok, curchar, oktochange)
+    unsigned char *	buf;
+    unsigned char *	start; 
+    register unsigned char *
+			tok;
+    unsigned char **	curchar;	/* Where to do insertion (updated) */
+    int			oktochange;	/* NZ if OK to modify tok */
     {
-    char		copy[BUFSIZ];
-    register char *	p;
-    register char *	q;
-    char *		ew;
+    unsigned char	copy[BUFSIZ];
+    register unsigned char *
+			p;
+    register unsigned char *
+			q;
+    unsigned char *	ew;
 
-    (void) strcpy (copy, buf);
+    (void) strcpy ((char *) copy, (char *) buf);
 
     for (p = buf, q = copy; p != start; p++, q++)
 	*p = *q;
@@ -687,7 +796,9 @@ static void inserttoken (buf, start, tok, curchar)
 	*/
 
 	*p++ = *tok;
-	*tok++ = '\0';
+	if (oktochange)
+	    *tok = '\0';
+	tok++;
 	while (*tok)
 	    *p++ = *tok++;
 	}
@@ -696,16 +807,16 @@ static void inserttoken (buf, start, tok, curchar)
     }
 
 static int posscmp (a, b)
-    char *		a;
-    char *		b;
+    unsigned char *	a;
+    unsigned char *	b;
     {
 
     return casecmp (a, b, 0);
     }
 
 int casecmp (a, b, canonical)
-    char *		a;
-    char *		b;
+    unsigned char *	a;
+    unsigned char *	b;
     int			canonical;	/* NZ for canonical string chars */
     {
     register ichar_t *	ap;
@@ -759,9 +870,7 @@ void makepossibilities (word)
     maxposslen = 0;
     easypossibilities = 0;
 
-#ifndef NO_CAPITALIZATION_SUPPORT
     wrongcapital (word);
-#endif
 
 /* 
  * according to Pollock and Zamora, CACM April 1984 (V. 27, No. 4),
@@ -805,17 +914,18 @@ static int insert (word)
     register ichar_t *	word;
     {
     register int	i;
-    register char *	realword;
+    register unsigned char *
+			realword;
 
     realword = ichartosstr (word, 0);
     for (i = 0; i < pcount; i++)
 	{
-	if (strcmp (possibilities[i], realword) == 0)
+	if (strcmp (possibilities[i], (char *) realword) == 0)
 	    return (0);
 	}
 
-    (void) strcpy (possibilities[pcount++], realword);
-    i = strlen (realword);
+    (void) strcpy (possibilities[pcount++], (char *) realword);
+    i = strlen ((char *) realword);
     if (i > maxposslen)
 	maxposslen = i;
     if (pcount >= MAXPOSSIBLE)
@@ -824,7 +934,6 @@ static int insert (word)
 	return (0);
     }
 
-#ifndef NO_CAPITALIZATION_SUPPORT
 static void wrongcapital (word)
     register ichar_t *	word;
     {
@@ -842,7 +951,6 @@ static void wrongcapital (word)
 	(void) ins_cap (newword, word);
 	}
     }
-#endif
 
 static void wrongletter (word)
     register ichar_t *	word;
@@ -855,9 +963,7 @@ static void wrongletter (word)
 
     n = icharlen (word);
     (void) icharcpy (newword, word);
-#ifndef NO_CAPITALIZATION_SUPPORT
     upcase (newword);
-#endif
 
     for (i = 0; i < n; i++)
 	{
@@ -1186,42 +1292,18 @@ static void save_root_cap (word, pattern, prestrip, preadd, sufstrip, sufadd,
 					/* Room to save words */
     int *		nsaved;		/* Number saved so far (updated) */
     {
-#ifndef NO_CAPITALIZATION_SUPPORT
     register struct dent * dent;
-#endif /* NO_CAPITALIZATION_SUPPORT */
     int			firstisupper;
     ichar_t		newword[INPUTWORDLEN + 4 * MAXAFFIXLEN + 4];
-#ifndef NO_CAPITALIZATION_SUPPORT
     register ichar_t *	p;
     int			len;
     int			i;
     int			limit;
-#endif /* NO_CAPITALIZATION_SUPPORT */
 
     if (*nsaved >= MAX_CAPS)
 	return;
     (void) icharcpy (newword, word);
     firstisupper = myupper (pattern[0]);
-#ifdef NO_CAPITALIZATION_SUPPORT
-    /*
-    ** Apply the old, simple-minded capitalization rules.
-    */
-    if (firstisupper)
-	{
-	if (myupper (pattern[1]))
-	    upcase (newword);
-	else
-	    {
-	    lowcase (newword);
-	    newword[0] = mytoupper (newword[0]);
-	    }
-	}
-    else
-	lowcase (newword);
-    (void) icharcpy (savearea[*nsaved], newword);
-    (*nsaved)++;
-    return;
-#else /* NO_CAPITALIZATION_SUPPORT */
 #define flagsareok(dent)    \
     ((pfxent == NULL \
 	||  TSTMASKBIT (dent->mask, pfxent->flagbit)) \
@@ -1377,7 +1459,6 @@ static void save_root_cap (word, pattern, prestrip, preadd, sufstrip, sufadd,
 	dent = dent->next;
 	}
     return;
-#endif /* NO_CAPITALIZATION_SUPPORT */
     }
 
 static char * getline (s)
@@ -1391,12 +1472,12 @@ static char * getline (s)
     for (  ;  ;  )
 	{
 	(void) fflush (stdout);
-	c = (GETKEYSTROKE () & NOPARITY);
+	c = GETKEYSTROKE ();
 	if (c == '\\')
 	    {
 	    (void) putchar ('\\');
 	    (void) fflush (stdout);
-	    c = (GETKEYSTROKE () & NOPARITY);
+	    c = GETKEYSTROKE ();
 	    backup ();
 	    (void) putchar (c);
 	    *p++ = (char) c;
@@ -1438,10 +1519,12 @@ static char * getline (s)
 
 void askmode ()
     {
-    int			bufsize;	/* Length of contextbufs[0] */
+    unsigned int	bufsize;	/* Length of contextbufs[0] */
     int			ch;		/* Next character read from input */
-    register char *	cp1;
-    register char *	cp2;
+    register unsigned char *
+			cp1;
+    register unsigned char *
+			cp2;
     ichar_t *		itok;		/* Ichar version of current word */
     int			hadnl;		/* NZ if \n was at end of line */
 
@@ -1459,6 +1542,8 @@ void askmode ()
     contextoffset = 0;
     while (1)
 	{
+	if (askverbose)
+	    (void) printf ("word: ");
 	(void) fflush (stdout);
 	/*
 	 * Only read in enough characters to fill half this buffer so that any
@@ -1466,16 +1551,20 @@ void askmode ()
 	 */
 	if (contextoffset == 0)
 	    {
-	    if (xgets (contextbufs[0], (sizeof contextbufs[0]) / 2, stdin)
+	    if (xgets ((char *) filteredbuf, (sizeof filteredbuf) / 2, stdin)
 	      == NULL)
 		break;
 	    }
 	else
 	    {
-	    if (fgets (contextbufs[0], (sizeof contextbufs[0]) / 2, stdin)
+	    if (fgets ((char *) filteredbuf, (sizeof filteredbuf) / 2, stdin)
 	      == NULL)
 		break;
 	    }
+	/*
+	 * Make a copy of the line in contextbufs[0] so copyout works.
+	 */
+	(void) strcpy ((char *) contextbufs[0], (char *) filteredbuf);
 	/*
 	 * If we didn't read to end-of-line, we may have ended the
 	 * buffer in the middle of a word.  So keep reading until we
@@ -1483,26 +1572,20 @@ void askmode ()
 	 * word. (or until the buffer is full, which fortunately isn't
 	 * all that likely).
 	 */
-	bufsize = strlen (contextbufs[0]);
-	if (contextbufs[0][bufsize - 1] == '\n')
+	bufsize = strlen ((char *) filteredbuf);
+	hadnl = filteredbuf[bufsize - 1] == '\n';
+	if (bufsize == (sizeof filteredbuf) / 2 - 1)
 	    {
-	    hadnl = 1;
-	    contextbufs[0][--bufsize] = '\0';
-	    }
-	else
-	    hadnl = 0;
-	if (bufsize == (sizeof contextbufs[0]) / 2 - 1)
-	    {
-	    ch = (unsigned char) contextbufs[0][bufsize - 1];
-	    while (bufsize < sizeof contextbufs[0] - 1
+	    ch = (unsigned char) filteredbuf[bufsize - 1];
+	    while (bufsize < sizeof filteredbuf - 1
 	      &&  (iswordch ((ichar_t) ch)  ||  isboundarych ((ichar_t) ch)
 	      ||  isstringstart (ch)))
 		{
 		ch = getc (stdin);
 		if (ch == EOF)
 		    break;
-		contextbufs[0][bufsize++] = (char) ch;
-		contextbufs[0][bufsize] = '\0';
+		filteredbuf[bufsize++] = (char) ch;
+		filteredbuf[bufsize] = '\0';
 		}
 	    }
 	/*
@@ -1513,59 +1596,86 @@ void askmode ()
 	** `~' followed by a filename sets parameters according to file name
 	** `^' causes rest of line to be checked after stripping 1st char
 	*/
-	if (contextoffset != 0)
+	if (askverbose  ||  contextoffset != 0)
 	    checkline (stdout);
 	else
 	    {
-	    if (contextbufs[0][0] == '*'  ||  contextbufs[0][0] == '@')
-		treeinsert(ichartosstr (strtosichar (contextbufs[0] + 1, 0), 1),
+	    if (filteredbuf[0] == '*'  ||  filteredbuf[0] == '@')
+		treeinsert(ichartosstr (strtosichar (filteredbuf + 1, 0), 1),
 		  ICHARTOSSTR_SIZE,
-		  contextbufs[0][0] == '*');
-	    else if (contextbufs[0][0] == '&')
+		  filteredbuf[0] == '*');
+	    else if (filteredbuf[0] == '&')
 		{
-		itok = strtosichar (contextbufs[0] + 1, 0);
+		itok = strtosichar (filteredbuf + 1, 0);
 		lowcase (itok);
 		treeinsert (ichartosstr (itok, 1), ICHARTOSSTR_SIZE, 1);
 		}
-	    else if (contextbufs[0][0] == '#')
+	    else if (filteredbuf[0] == '#')
 		{
 		treeoutput ();
+		insidehtml = 0;
 		math_mode = 0;
 		LaTeX_Mode = 'P';
 		}
-	    else if (contextbufs[0][0] == '!')
+	    else if (filteredbuf[0] == '!')
 		terse = 1;
-	    else if (contextbufs[0][0] == '%')
+	    else if (filteredbuf[0] == '%')
+		{
 		terse = 0;
-	    else if (contextbufs[0][0] == '-')
+		correct_verbose_mode = 0;
+		}
+	    else if (filteredbuf[0] == '-')
 		{
+		insidehtml = 0;
 		math_mode = 0;
 		LaTeX_Mode = 'P';
-		tflag = 0;
+		tflag = DEFORMAT_NROFF;
 		}
-	    else if (contextbufs[0][0] == '+')
+	    else if (filteredbuf[0] == '+')
 		{
+		insidehtml = 0;
 		math_mode = 0;
 		LaTeX_Mode = 'P';
-		tflag = strcmp (&contextbufs[0][1], "nroff") != 0
-		  &&  strcmp (&contextbufs[0][1], "troff") != 0;
+		if (strcmp ((char *) &filteredbuf[1], "plain") == 0
+		  ||  strcmp ((char *) &filteredbuf[1], "none") == 0)
+		    tflag = DEFORMAT_NONE;
+		else if (strcmp ((char *) &filteredbuf[1], "nroff") == 0
+		  ||  strcmp ((char *) &filteredbuf[1], "troff") == 0)
+		    tflag = DEFORMAT_NROFF;
+		else if (strcmp ((char *) &filteredbuf[1], "tex") == 0
+		  ||  strcmp ((char *) &filteredbuf[1], "latex") == 0
+		  ||  filteredbuf[1] == '\0') /* Backwards compatibility */
+		    tflag = DEFORMAT_TEX;
+		else if (strcmp ((char *) &filteredbuf[1], "html") == 0
+		  ||  strcmp ((char *) &filteredbuf[1], "sgml") == 0)
+		    tflag = DEFORMAT_SGML;
+		else
+		    tflag = DEFORMAT_TEX;	/* Backwards compatibility */
 		}
-	    else if (contextbufs[0][0] == '~')
+	    else if (filteredbuf[0] == '~')
 		{
-		defdupchar = findfiletype (&contextbufs[0][1], 1, (int *) NULL);
+		if (hadnl)
+		    filteredbuf[bufsize - 1] = '\0';
+		defdupchar =
+		  findfiletype ((char *) &filteredbuf[1], 1, (int *) NULL);
 		if (defdupchar < 0)
 		    defdupchar = 0;
+		if (hadnl)
+		    filteredbuf[bufsize - 1] = '\n';
 		}
+	    else if (filteredbuf[0] == '`')
+		correct_verbose_mode = 1;
 	    else
 		{
-		if (contextbufs[0][0] == '^')
+		if (filteredbuf[0] == '^')
 		    {
 		    /* Strip off leading uparrow */
-		    for (cp1 = contextbufs[0], cp2 = contextbufs[0] + 1;
+		    for (cp1 = filteredbuf, cp2 = filteredbuf + 1;
 		      (*cp1++ = *cp2++) != '\0';
 		      )
 			;
 		    contextoffset++;
+		    bufsize--;
 		    }
 		checkline (stdout);
 		}
@@ -1586,31 +1696,45 @@ void askmode ()
 	    }
 #endif
 	}
+    if (askverbose)
+	(void) printf ("\n");
     }
 
-/* Copy/ignore "cnt" number of characters pointed to by *cc. */
+/*
+ * Copy up to "cnt" characters to the output file.  For historical
+ * reasons, cc points to a characer in "filteredbuf", but the copying
+ * must be done from "contextbufs[0]".  As a side effect this function
+ * advances cc by the number of characters actually copied.
+ */
 void copyout (cc, cnt)
-    register char **	cc;
-    register int	cnt;
+    unsigned char **	cc;		/* Char in filteredbuf to start at */
+    register int	cnt;		/* Number of chars to copy */
     {
-
+    register char *	cp;		/* Char in contextbufs[0] to copy */
+    
+    cp = &contextbufs[0][*cc - filteredbuf];
+    *cc += cnt;
     while (--cnt >= 0)
 	{
-	if (**cc == '\0')
+	if (*cp == '\0')
+	    {
+	    *cc -= cnt + 1;		/* Compensate for short copy */
 	    break;
+	    }
 	if (!aflag && !lflag)
-	    (void) putc (**cc, outfile);
-	(*cc)++;
+	    (void) putc (*cp, outfile);
+	cp++;
 	}
     }
 
 static void lookharder (string)
-    char *		string;
+    unsigned char *	string;
     {
     char		cmd[150];
     char		grepstr[100];
     register char *	g;
-    register char *	s;
+    register unsigned char *
+			s;
 #ifndef REGEX_LOOKUP
     register int	wild = 0;
 #ifdef LOOK
