@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: dump.c,v 1.14 1994/01/25 07:11:27 geoff Exp $";
+    "$Id: dump.c,v 1.16 1999/01/18 03:28:30 geoff Exp $";
 #endif
 
 /*
@@ -9,7 +9,7 @@ static char Rcs_Id[] =
  * This code originally resided in ispell.c, but was moved here to keep
  * file sizes smaller.
  *
- * Copyright 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * Copyright 1992, 1993, 1999, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,6 +48,13 @@ static char Rcs_Id[] =
 
 /*
  * $Log: dump.c,v $
+ * Revision 1.16  1999/01/18 03:28:30  geoff
+ * Turn many char declarations into unsigned char, so that we won't have
+ * sign-extension problems.
+ *
+ * Revision 1.15  1999/01/07  01:57:56  geoff
+ * Update the copyright.
+ *
  * Revision 1.14  1994/01/25  07:11:27  geoff
  * Get rid of all old RCS log lines in preparation for the 3.1 release.
  *
@@ -119,8 +126,9 @@ static void entdump (flagp)		/* Dump one flag entry */
     (void) printf ("\t> ");
     (void) putc ('\t', stdout);
     if (flagp->stripl)
-	(void) printf ("-%s,", ichartosstr (flagp->strip, 1));
-    (void) printf ("%s\n", flagp->affl ? ichartosstr (flagp->affix, 1) : "-");
+	(void) printf ("-%s,", (char *) ichartosstr (flagp->strip, 1));
+    (void) printf ("%s\n",
+      flagp->affl ? (char*) ichartosstr (flagp->affix, 1) : "-");
     }
 
 static void setdump (setp, mask)	/* Dump a set specification */
