@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: icombine.c,v 2.24 1994/01/25 07:11:35 geoff Exp $";
+    "$Id: icombine.c,v 2.28 2000/08/22 10:52:25 geoff Exp $";
 #endif
 
 #define MAIN
@@ -15,7 +15,7 @@ static char Rcs_Id[] =
  * to buildhash and ispell, we can be sure that the rules for combining
  * capitalizations are compatible.
  *
- * Copyright 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * Copyright 1992, 1993, 1999, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,6 +54,18 @@ static char Rcs_Id[] =
 
 /*
  * $Log: icombine.c,v $
+ * Revision 2.28  2000/08/22 10:52:25  geoff
+ * Fix some compiler warnings.
+ *
+ * Revision 2.27  1999/01/07 01:22:44  geoff
+ * Update the copyright.
+ *
+ * Revision 2.26  1999/01/03  01:46:30  geoff
+ * Add support for sgml and plain deformatter types.
+ *
+ * Revision 2.25  1997/12/02  06:24:45  geoff
+ * Get rid of some compile options that really shouldn't be optional.
+ *
  * Revision 2.24  1994/01/25  07:11:35  geoff
  * Get rid of all old RCS log lines in preparation for the 3.1 release.
  *
@@ -121,8 +133,10 @@ int main (argc, argv)
 	{
 	defdupchar = findfiletype (preftype, 1, (int *) NULL);
 	if (defdupchar < 0
+	  &&  strcmp (preftype, "plain") != 0
 	  &&  strcmp (preftype, "tex") != 0
-	  &&  strcmp (preftype, "nroff") != 0)
+	  &&  strcmp (preftype, "nroff") != 0
+	  &&  strcmp (preftype, "sgml") != 0)
 	    {
 	    (void) fprintf (stderr, ICOMBINE_C_BAD_TYPE, preftype);
 	    exit (1);
@@ -169,7 +183,7 @@ static void combinedict ()
     {
     struct dent		d;
     register struct dent * dp;
-    char		lbuf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
+    unsigned char	lbuf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
     ichar_t		ucbuf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
     ichar_t		lastbuf[INPUTWORDLEN + MAXAFFIXLEN + 2 * MASKBITS];
 
@@ -177,7 +191,7 @@ static void combinedict ()
     hashtbl = (struct dent *) mymalloc (sizeof (struct dent));
     hashtbl->flagfield = 0;
     hashtbl->word = 0;
-    while (fgets (lbuf, sizeof lbuf, stdin) != NULL)
+    while (fgets ((char *) lbuf, sizeof lbuf, stdin) != NULL)
 	{
 	if (ichartostr (lbuf, strtosichar (lbuf, 0), sizeof lbuf, 1))
 	    (void) fprintf (stderr, WORD_TOO_LONG (lbuf));
@@ -202,7 +216,6 @@ static void combinedict ()
 	if ((dp->flagfield & USED) == 0)
 	    {
 	    *dp = d;
-#ifndef NO_CAPITALIZATION_SUPPORT
 	    /*
 	    ** If it's a followcase word, we need to make this a
 	    ** special dummy entry, and add a second with the
@@ -213,7 +226,6 @@ static void combinedict ()
 		if (addvheader (dp))
 		  exit (1);
 		}
-#endif
 	    }
 	else
 	    {
@@ -244,7 +256,6 @@ static void combineout ()
 	    toutent (stdout, tdp, 0);
 	    myfree (tdp->word);
 	    ndp = tdp->next;
-#ifndef NO_CAPITALIZATION_SUPPORT
 	    while (tdp->flagfield & MOREVARIANTS)
 		{
 		if (tdp != hashtbl)
@@ -254,7 +265,6 @@ static void combineout ()
 		    myfree (tdp->word);
 		ndp = tdp->next;
 		}
-#endif
 	    if (tdp != hashtbl)
 		myfree ((char *) tdp);
 	    }
