@@ -1,10 +1,10 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: ijoin.c,v 1.6 1994/10/18 04:03:21 geoff Exp $";
+    "$Id: ijoin.c,v 1.9 2001/06/07 19:06:33 geoff Exp $";
 #endif
 
 /*
- * Copyright 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * Copyright 1992, 1993, 1999, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -78,6 +78,16 @@ static char Rcs_Id[] =
 
 /*
  * $Log: ijoin.c,v $
+ * Revision 1.9  2001/06/07 19:06:33  geoff
+ * Add a workaround for the strcmp-as-macro problem found on recent
+ * releases of gcc.
+ *
+ * Revision 1.8  2000/08/22 10:52:25  geoff
+ * Fix some compiler warnings.
+ *
+ * Revision 1.7  1999/01/07 01:58:00  geoff
+ * Update the copyright.
+ *
  * Revision 1.6  1994/10/18  04:03:21  geoff
  * Fix a couple of bugs, one where the last field on a line could be
  * output incorrectly, and one where fields from the wrong file could be
@@ -119,6 +129,16 @@ typedef struct
     }
 		outlist_t;		/* Output description list */
 
+/*
+ * Some excessively "smart" compilers (for which you are welcome to
+ * read "stupid compiler implementers") try to get clever about
+ * strcmp, turning it into a macro in some circumstances.  That breaks
+ * things, since you then can't take the address of what is supposed
+ * to be a library function.  To be safe, we'll #undef any possible
+ * macro here to get around that particular lack of foresight.
+ */
+#undef strcmp
+
 static int		(*compare) () = strcmp; /* Comparison function */
 static char *		emptyfield = ""; /* Use this to replace empty fields */
 static FILE *		file1;		/* First file to join */
@@ -271,9 +291,9 @@ static void dojoin ()			/* Actually perform the join */
 	 * Do a little work to reduce the number of calls to realloc, at
 	 * the expense of slightly-increased memory usage.
 	 */
-	if (line1 != NULL  &&  line1->nfields >= field_field_inc)
+	if (line1 != NULL  &&  line1->nfields >= (int) field_field_inc)
 	    field_field_inc = line1->nfields + 1;
-	if (line2 != NULL  &&  line2->nfields >= field_field_inc)
+	if (line2 != NULL  &&  line2->nfields >= (int) field_field_inc)
 	    field_field_inc = line2->nfields + 1;
 	/*
 	 * Determine if the lines can be paired.
