@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: ispell.c,v 1.133 1995/10/11 04:30:29 geoff Exp $";
+    "$Id: ispell.c,v 1.148 2001/05/30 21:14:47 geoff Exp $";
 #endif
 
 #define MAIN
@@ -10,7 +10,7 @@ static char Rcs_Id[] =
  *
  * Copyright (c), 1983, by Pace Willisson
  *
- * Copyright 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * Copyright 1992, 1993, 1999, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,6 +49,59 @@ static char Rcs_Id[] =
 
 /*
  * $Log: ispell.c,v $
+ * Revision 1.148  2001/05/30 21:14:47  geoff
+ * Invert the fcntl/mkstemp options so they will default to being used.
+ *
+ * Revision 1.147  2001/05/30 21:04:25  geoff
+ * Add fcntl.h support (needed for the previous change), security
+ * commentary, and mkstemp support.
+ *
+ * Revision 1.146  2001/05/30 09:38:26  geoff
+ * Werner Fink's patch to get rid of a temp-file race condition.
+ *
+ * Revision 1.145  2001/05/07 04:47:18  geoff
+ * Flush stdout after expansion, for use in pipes (Ed Avis)
+ *
+ * Revision 1.144  2000/08/22 10:52:25  geoff
+ * Fix some compiler warnings.
+ *
+ * Revision 1.143  1999/01/13 01:34:21  geoff
+ * Get rid of some obsolete variables in the -vv switch.
+ *
+ * Revision 1.142  1999/01/08  05:45:03  geoff
+ * Allow shtml files to force HTML mode.
+ *
+ * Revision 1.141  1999/01/07  01:22:47  geoff
+ * Update the copyright.
+ *
+ * Revision 1.140  1999/01/03  01:46:35  geoff
+ * Add support for external deformatters.
+ *
+ * Revision 1.139  1998/07/12  20:42:18  geoff
+ * Change the -i switch to -k, and make it take the name of a keyword
+ * table.
+ *
+ * Revision 1.138  1998/07/06  06:55:16  geoff
+ * Use the new general-keyword-lookup support to initialize HTML and TEX
+ * tags from environment variables or defaults.
+ *
+ * Revision 1.137  1997/12/02  06:24:48  geoff
+ * Get rid of some compile options that really shouldn't be optional.
+ * Add HTML support ('i' switch).
+ *
+ * Revision 1.136  1995/11/08  05:09:14  geoff
+ * Set "aflag" and "askverbose" (new interactive mode) if invoked without
+ * filenames or mode arguments.
+ *
+ * Revision 1.135  1995/11/08  04:27:59  geoff
+ * Improve the HTML support to interoperate better with nroff/troff, clean
+ * it up, and use '-H' for the switch.
+ *
+ * Revision 1.134  1995/10/25  04:05:23  geoff
+ * Modifications made by Gerry Tierney <gtierney@nova.ucd.ie> to allow
+ * checking of html code.  Adds -h switch and checking for html files by
+ * .html or .htm extension.  14th of October 1995.
+ *
  * Revision 1.133  1995/10/11  04:30:29  geoff
  * Get rid of an unused variable.
  *
@@ -133,12 +186,16 @@ static char Rcs_Id[] =
 #include "msgs.h"
 #include "version.h"
 #include <ctype.h>
+#ifndef NO_FCNTL_H
+#include <fcntl.h>
+#endif /* NO_FCNTL_H */
 #include <sys/stat.h>
 
 static void	usage P ((void));
 static void	initckch P ((char * wchars));
 int		main P ((int argc, char * argv[]));
 static void	dofile P ((char * filename));
+static FILE *	setupdefmt P ((char * filename));
 static void	update_file P ((char * filename, struct stat * statbuf));
 static void	expandmode P ((int printorig));
 
@@ -230,7 +287,7 @@ static void initckch (wchars)
 			}
 		    }
 		}
-	    c &= NOPARITY;
+	    c &= 0xFF;
 	    if (!hashheader.wordchars[c])
 		{
 		hashheader.wordchars[c] = 1;
@@ -296,9 +353,9 @@ int main (argc, argv)
 	 * Used:
 	 *
 	 *	ABCDEFGHIJKLMNOPQRSTUVWXYZ 0123456789
-	 *	^^^^       ^^^ ^  ^^ ^^
+	 *	^^^^ ^ ^   ^^^ ^  ^^ ^^
 	 *	abcdefghijklmnopqrstuvwxyz
-	 *	^^^^^^     ^^^ ^  ^^ ^^^
+	 *	^^^^^^  ^   ^^^ ^ ^^^ ^^^
 	 */
 	arglen = strlen (*argv);
 	switch ((*argv)[1])
@@ -354,8 +411,6 @@ int main (argc, argv)
 		    (void) printf ("\tDEFPDICT = \"%s\"\n", DEFPDICT);
 		    (void) printf ("\tDEFTEXFLAG = %d\n", DEFTEXFLAG);
 		    (void) printf ("\tEGREPCMD = \"%s\"\n", EGREPCMD);
-		    (void) printf ("\tELISPDIR = \"%s\"\n", ELISPDIR);
-		    (void) printf ("\tEMACS = \"%s\"\n", EMACS);
 #ifdef EQUAL_COLUMNS
 		    (void) printf ("\tEQUAL_COLUMNS\n");
 #else /* EQUAL_COLUMNS */
@@ -424,21 +479,11 @@ int main (argc, argv)
 		    (void) printf ("\tMSDOS_BINARY_OPEN = 0x%x\n",
 		      (unsigned int) MSDOS_BINARY_OPEN);
 		    (void) printf ("\tMSGLANG = %s\n", MSGLANG);
-#ifdef NO_CAPITALIZATION_SUPPORT
-		    (void) printf ("\tNO_CAPITALIZATION_SUPPORT\n");
-#else /* NO_CAPITALIZATION_SUPPORT */
-		    (void) printf ("\t!NO_CAPITALIZATION_SUPPORT\n");
-#endif /* NO_CAPITALIZATION_SUPPORT */
 #ifdef NO_STDLIB_H
 		    (void) printf ("\tNO_STDLIB_H\n");
 #else /* NO_STDLIB_H */
 		    (void) printf ("\t!NO_STDLIB_H (STDLIB_H)\n");
 #endif /* NO_STDLIB_H */
-#ifdef NO8BIT
-		    (void) printf ("\tNO8BIT\n");
-#else /* NO8BIT */
-		    (void) printf ("\t!NO8BIT (8BIT)\n");
-#endif /* NO8BIT */
 		    (void) printf ("\tNRSPECIAL = \"%s\"\n", NRSPECIAL);
 		    (void) printf ("\tOLDPAFF = \"%s\"\n", OLDPAFF);
 		    (void) printf ("\tOLDPDICT = \"%s\"\n", OLDPDICT);
@@ -467,7 +512,6 @@ int main (argc, argv)
 		    (void) printf ("\tSTATSUFFIX = \"%s\"\n", STATSUFFIX);
 		    (void) printf ("\tTEMPNAME = \"%s\"\n", TEMPNAME);
 		    (void) printf ("\tTERMLIB = \"%s\"\n", TERMLIB);
-		    (void) printf ("\tTEXINFODIR = \"%s\"\n", TEXINFODIR);
 		    (void) printf ("\tTEXSPECIAL = \"%s\"\n", TEXSPECIAL);
 #ifdef TRUNCATEBAK
 		    (void) printf ("\tTRUNCATEBAK\n");
@@ -487,30 +531,86 @@ int main (argc, argv)
 	    case 'n':
 		if (arglen > 2)
 		    usage ();
-		tflag = 0;		/* nroff/troff mode */
-		deftflag = 0;
+		tflag = DEFORMAT_NROFF;	/* nroff/troff mode */
+		deftflag = DEFORMAT_NROFF;
 		if (preftype == NULL)
 		    preftype = "nroff";
 		break;
 	    case 't':			/* TeX mode */
 		if (arglen > 2)
 		    usage ();
-		tflag = 1;
-		deftflag = 1;
+		tflag = DEFORMAT_TEX;
+		deftflag = DEFORMAT_TEX;
 		if (preftype == NULL)
 		    preftype = "tex";
 		break;
+	    case 'H':			/* HTML mode */
+		if (arglen > 2)
+		    usage ();
+		tflag = DEFORMAT_SGML;	/* non-TeX mode */
+		deftflag = DEFORMAT_SGML;
+		if (preftype == NULL)
+		    preftype = "sgml";
+		break;
+	    case 'k':			/* Set keyword tables */
+		p = (*argv) + 2;
+		argv++;
+		argc--;
+		if (argc == 0)
+		    usage ();
+		if (strcmp (p, "texskip1") == 0)
+		    {
+		    if (init_keyword_table (*argv, TEXSKIP1VAR, TEXSKIP1,
+		      0, &texskip1list))
+			usage ();
+		    }
+		else if (strcmp (p, "texskip2") == 0)
+		    {
+		    if (init_keyword_table (*argv, TEXSKIP2VAR, TEXSKIP2,
+		      0, &texskip2list))
+			usage ();
+		    }
+		else if (strcmp (p, "htmlignore") == 0)
+		    {
+		    if (init_keyword_table (*argv, HTMLIGNOREVAR, HTMLIGNORE,
+		      1, &htmlignorelist))
+			usage ();
+		    }
+		else if (strcmp (p, "htmlcheck") == 0)
+		    {
+		    if (init_keyword_table (*argv, HTMLCHECKVAR, HTMLCHECK,
+		      1, &htmlchecklist))
+			usage ();
+		    }
+		break;
 	    case 'T':			/* Set preferred file type */
-		p = (*argv)+2;
+		p = (*argv) + 2;
 		if (*p == '\0')
 		    {
-		    argv++; argc--;
+		    argv++;
+		    argc--;
 		    if (argc == 0)
 			usage ();
 		    p = *argv;
 		    }
 		preftype = p;
 		break;
+	    case 'F':			/* Set external deformatting program */
+		p = (*argv) + 2;
+		if (*p == '\0')
+		    {
+		    argv++;
+		    argc--;
+		    if (argc == 0)
+			usage ();
+		    p = *argv;
+		    }
+		defmtpgm = p;
+		tflag = DEFORMAT_NONE;
+		deftflag = DEFORMAT_NONE;
+		if (preftype == NULL)
+		    preftype = "plain";
+		break;
 	    case 'A':
 		if (arglen > 2)
 		    usage ();
@@ -559,10 +659,11 @@ int main (argc, argv)
 		break;
 	    case 'f':
 		fflag++;
-		p = (*argv)+2;
+		p = (*argv) + 2;
 		if (*p == '\0')
 		    {
-		    argv++; argc--;
+		    argv++;
+		    argc--;
 		    if (argc == 0)
 			usage ();
 		    p = *argv;
@@ -572,10 +673,11 @@ int main (argc, argv)
 		    askfilename = NULL;
 		break;
 	    case 'L':
-		p = (*argv)+2;
+		p = (*argv) + 2;
 		if (*p == '\0')
 		    {
-		    argv++; argc--;
+		    argv++;
+		    argc--;
 		    if (argc == 0)
 			usage ();
 		    p = *argv;
@@ -630,10 +732,11 @@ int main (argc, argv)
 		minimenusize = 2;
 		break;
 	    case 'p':
-		cpd = (*argv)+2;
+		cpd = (*argv) + 2;
 		if (*cpd == '\0')
 		    {
-		    argv++; argc--;
+		    argv++;
+		    argc--;
 		    if (argc == 0)
 			usage ();
 		    cpd = *argv;
@@ -643,10 +746,11 @@ int main (argc, argv)
 		LibDict = NULL;
 		break;
 	    case 'd':
-		p = (*argv)+2;
+		p = (*argv) + 2;
 		if (*p == '\0')
 		    {
-		    argv++; argc--;
+		    argv++;
+		    argc--;
 		    if (argc == 0)
 			usage ();
 		    p = *argv;
@@ -675,10 +779,11 @@ int main (argc, argv)
 		vflag = 1;
 		break;
 	    case 'w':
-		wchars = (*argv)+2;
+		wchars = (*argv) + 2;
 		if (*wchars == '\0')
 		    {
-		    argv++; argc--;
+		    argv++;
+		    argc--;
 		    if (argc == 0)
 			usage ();
 		    wchars = *argv;
@@ -687,7 +792,8 @@ int main (argc, argv)
 	    case 'W':
 		if ((*argv)[2] == '\0')
 		    {
-		    argv++; argc--;
+		    argv++;
+		    argc--;
 		    if (argc == 0)
 			usage ();
 		    minword = atoi (*argv);
@@ -703,7 +809,15 @@ int main (argc, argv)
 	}
 
     if (!argc  &&  !lflag  &&  !aflag   &&  !eflag  &&  !dumpflag)
-	usage ();
+	{
+	if (argc != 0)
+	    usage ();
+	else
+	    {
+	    aflag = 1;
+	    askverbose = 1;
+	    }
+	}
 
     /*
      * Because of the high cost of reading the dictionary, we stat
@@ -729,8 +843,10 @@ int main (argc, argv)
 	prefstringchar =
 	  findfiletype (preftype, 1, deftflag < 0 ? &deftflag : (int *) NULL);
 	if (prefstringchar < 0
+	  &&  strcmp (preftype, "plain") != 0
 	  &&  strcmp (preftype, "tex") != 0
-	  &&  strcmp (preftype, "nroff") != 0)
+	  &&  strcmp (preftype, "nroff") != 0
+	  &&  strcmp (preftype, "sgml") != 0)
 	    {
 	    (void) fprintf (stderr, ISPELL_C_BAD_TYPE, preftype);
 	    exit (1);
@@ -746,6 +862,21 @@ int main (argc, argv)
     if (tryhardflag < 0)
 	tryhardflag = hashheader.defhardflag;
 
+    /*
+     * Set up the various tables of keywords to be treated specially.
+     *
+     * TeX/LaTeX mode:
+     */
+    (void) init_keyword_table (NULL, TEXSKIP1VAR, TEXSKIP1, 0, &texskip1list);
+    (void) init_keyword_table (NULL, TEXSKIP2VAR, TEXSKIP2, 0, &texskip2list);
+    /*
+     * HTML mode:
+     */
+    (void) init_keyword_table (NULL, HTMLIGNOREVAR, HTMLIGNORE, 1,
+      &htmlignorelist);
+    (void) init_keyword_table (NULL, HTMLCHECKVAR, HTMLCHECK, 1,
+      &htmlchecklist);
+
     initckch(wchars);
 
     if (LibDict == NULL)	
@@ -781,7 +912,7 @@ int main (argc, argv)
 #endif /* __bsdi__ */
     if (lflag)
 	{
-	infile = stdin;
+	infile = setupdefmt(NULL);
 	outfile = stdout;
 	checkfile ();
 	exit (0);
@@ -802,15 +933,30 @@ static void dofile (filename)
     {
     struct stat	statbuf;
     char *	cp;
+    int		outfd;			/* Used in opening temp file */
+					/* ..might produce not-used warnings */
 
     currentfile = filename;
 
-    /* See if the file is a .tex file.  If so, set the appropriate flags. */
+    /* Guess a deformatter based on the file extension */
     tflag = deftflag;
     if (tflag < 0)
-	tflag =
-	  (cp = rindex (filename, '.')) != NULL  &&  strcmp (cp, ".tex") == 0;
-
+	{
+	tflag = DEFORMAT_NONE;		/* Default to none */
+	cp = rindex (filename, '.');
+	if (cp != NULL)
+	    {
+	    if (strcmp (cp, ".ms") == 0  ||  strcmp (cp, ".mm") == 0
+	      ||  strcmp (cp, ".me") == 0  ||  strcmp (cp, ".man") == 0
+	      ||  isdigit(*cp))
+		tflag = DEFORMAT_NROFF;
+	    else if (strcmp (cp, ".tex") == 0)
+		tflag = DEFORMAT_TEX;
+	    else if (strcmp (cp, ".html") == 0  ||  strcmp (cp, ".htm") == 0
+	      ||  strcmp (cp, ".shtml") == 0)
+		tflag = DEFORMAT_SGML;
+	    }
+	}
     if (prefstringchar < 0)
 	{
 	defdupchar =
@@ -819,7 +965,7 @@ static void dofile (filename)
 	    defdupchar = 0;
 	}
 
-    if ((infile = fopen (filename, "r")) == NULL)
+    if ((infile = setupdefmt (filename)) == NULL)
 	{
 	(void) fprintf (stderr, CANT_OPEN, filename);
 	(void) sleep ((unsigned) 2);
@@ -834,9 +980,26 @@ static void dofile (filename)
 	}
 
     (void) fstat (fileno (infile), &statbuf);
+    /*
+     * Security notes: TEMPNAME must be less than MAXPATHLEN - 1.  If
+     * the system has O_EXCL but not mkstemp, the temporary file will
+     * be opened securely as in the manner of mkstemp (which
+     * unfortunately isn't available anywhere).  In other words, don't
+     * worry about the security of this hunk of code.
+     */
     (void) strcpy (tempfile, TEMPNAME);
+#ifdef NO_MKSTEMP
     if (mktemp (tempfile) == NULL  ||  tempfile[0] == '\0'
+#ifdef O_EXCL
+      ||  (outfd = open (tempfile, O_WRONLY | O_CREAT | O_EXCL, 0600)) < 0
+      ||  (outfile = fdopen (outfd, "w")) == NULL)
+#else /* O_EXCL */
       ||  (outfile = fopen (tempfile, "w")) == NULL)
+#endif /* O_EXCL */
+#else /* NO_MKSTEMP */
+    if ((outfd = mkstemp (tempfile)) < 0
+      ||  (outfile = fdopen (outfd, "w")) == NULL)
+#endif /* NO_MKSTEMP */
 	{
 	(void) fprintf (stderr, CANT_CREATE,
 	  (tempfile == NULL  ||  tempfile[0] == '\0')
@@ -862,6 +1025,95 @@ static void dofile (filename)
     (void) unlink (tempfile);
     }
 
+/*
+ * Set up to externally deformat a file.
+ *
+ * If no deformatter is provided, we return either standard input (if
+ * filename is NULL) or a the result of opening the specified file.
+ *
+ * If a deformatter is provided, but no filename is provided, we
+ * assume that the input comes from stdin, and we set up the
+ * deformatter to filter that descriptor, and return stdin as our
+ * result.  In such a case, there is no dual input: we will read the
+ * filtered data via the deformatter and ignore the unfiltered
+ * version.
+ *
+ * If there is a deformatter and we are given a filename, then we set
+ * up the deformatter to read the file from its own standard input and
+ * write to its standard output.  In addition, we set up "sourcefile"
+ * so that it can read the unfiltered data directly from the input
+ * file.
+ *
+ * Note that external deformatters do not work with named pipes as input.
+ */
+static FILE * setupdefmt (filename)
+    char *		filename;	/* File to open, if non-NULL */
+    {
+    FILE*		filteredfile;	/* Access to the filtered file */
+    int			inputfd;	/* Fd for access to file to open */
+    int			savedstdin;	/* File descriptor saving stdin */
+
+    sourcefile = NULL;
+    if (defmtpgm == NULL)
+	{
+	/*
+	 * There is no deformatter.  Return either stdin or an open file.
+	 */
+	if (filename == NULL)
+	    return stdin;
+	else
+	    return fopen (filename, "r");
+	}
+    else if (filename == NULL)
+	{
+	/*
+	 * We are reading from standard input.  Switch over to a
+	 * filtered version of stdin.
+	 */
+	return popen (defmtpgm, "r");
+	}
+    else
+	{
+	/*
+	 * This is the tricky case.  We need to get the deformatter to
+	 * read from the input file and filter it to us.  Doing so
+	 * requires several steps:
+	 *
+	 *  1.	Preserve file descriptor 0 by duplicating it.
+	 *  2.	Close file descriptor 0, and reopen it on the file to be
+	 *	filtered.
+	 *  3.	Open a pipe to the deformat program.
+	 *  4.	Restore file descriptor 0.
+	 *
+	 * Because we do all of this without ever letting the stdio
+	 * library know that we've been mucking around with file
+	 * descriptors, we won't bother its access to stdin.
+	 */
+	sourcefile = fopen (filename, "r");
+	if (sourcefile == NULL)
+	    return NULL;
+	savedstdin = dup (0);
+	close (0);
+	inputfd = open (filename, 0);
+	if (inputfd < 0)
+	    return NULL;		/* Failed to open the file */
+	else if (inputfd != 0)
+	    {
+	    (void) fprintf (stderr, ISPELL_C_UNEXPECTED_FD, filename);
+	    exit (1);
+	    }
+	filteredfile = popen (defmtpgm, "r");
+	close (0);
+	if (dup (savedstdin) != 0)
+	    {
+	    (void) fprintf (stderr, ISPELL_C_UNEXPECTED_FD, filename);
+	    exit (1);
+	    }
+	close (savedstdin);
+	return filteredfile;
+	}
+    }
+
 static void update_file (filename, statbuf)
     char *		filename;
     struct stat *	statbuf;
@@ -987,20 +1239,25 @@ static void expandmode (option)
 		if (*flagp == hashheader.flagmarker)
 		    flagp++;
 		}
-	    if (strtoichar (ibuf, buf, sizeof ibuf, 1))
+	    if (strtoichar (ibuf, (unsigned char *) buf, sizeof ibuf, 1))
 		(void) fprintf (stderr, WORD_TOO_LONG (buf));
-	    explength = expand_pre (origbuf, ibuf, mask, option, "");
-	    explength += expand_suf (origbuf, ibuf, mask, 0, option, "");
+	    explength = expand_pre ((unsigned char *) origbuf, ibuf, mask,
+	      option, (unsigned char *) "");
+	    explength += expand_suf ((unsigned char *) origbuf, ibuf, mask, 0,
+	      option, (unsigned char *) "");
 	    explength += rootlength;
 	    if (option == 4)
 		{
 		(void) sprintf (ratiobuf, " %f",
 		  (double) explength / (double) rootlength);
 		(void) fputs (ratiobuf, stdout);
-		(void) expand_pre (origbuf, ibuf, mask, 3, ratiobuf);
-		(void) expand_suf (origbuf, ibuf, mask, 0, 3, ratiobuf);
+		(void) expand_pre ((unsigned char *) origbuf, ibuf, mask, 3,
+		  (unsigned char *) ratiobuf);
+		(void) expand_suf ((unsigned char *) origbuf, ibuf, mask, 0, 3,
+		  (unsigned char *) ratiobuf);
 		}
 	    }
 	(void) putchar ('\n');
+	(void) fflush (stdout);
 	}
     }
