@@ -1,9 +1,9 @@
 /*
- * $Id: ispell.h,v 1.67 1995/01/03 19:24:12 geoff Exp $
+ * $Id: ispell.h,v 1.78 2000/08/22 10:52:25 geoff Exp $
  */
 
 /*
- * Copyright 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * Copyright 1992, 1993, 1999, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,6 +42,45 @@
 
 /*
  * $Log: ispell.h,v $
+ * Revision 1.78  2000/08/22 10:52:25  geoff
+ * Fix a whole bunch of signed/unsigned discrepancies.
+ *
+ * Revision 1.77  2000/08/22 00:11:25  geoff
+ * Add support for correct_verbose_mode.
+ *
+ * Revision 1.76  1999/01/18 02:14:09  geoff
+ * Change most char declarations to unsigned char, to avoid
+ * sign-extension problems with 8-bit characters.
+ *
+ * Revision 1.75  1999/01/07  01:23:05  geoff
+ * Update the copyright.
+ *
+ * Revision 1.74  1999/01/03  01:46:37  geoff
+ * Add support for external deformatters.
+ *
+ * Revision 1.73  1998/07/06  06:55:19  geoff
+ * Add struct kwtable and some new variables, to support the new
+ * generalized keyword-lookup stuff.
+ *
+ * Revision 1.72  1997/12/02  06:24:54  geoff
+ * Get rid of some compile options that really shouldn't be optional.
+ *
+ * Revision 1.71  1997/12/01  00:53:49  geoff
+ * Add HTML support variables.
+ *
+ * Revision 1.70  1995/11/08  05:09:18  geoff
+ * Add the DEFORMAT_xxx constants and askverbose.  Improve the stylistic
+ * consistency of the HTML support.
+ *
+ * Revision 1.69  1995/10/25  04:05:26  geoff
+ * Patch by Gerry Tierney <gtierney@nova.ucd.ie> 1995/10/14.  Added
+ * variables htmlflag and insidehtml for use in html-mode.
+ *
+ * Revision 1.68  1995/03/06  02:42:41  geoff
+ * Be vastly more paranoid about parenthesizing macro arguments.  This
+ * fixes a bug in defmt.c where a complex argument was passed to
+ * isstringch.
+ *
  * Revision 1.67  1995/01/03  19:24:12  geoff
  * Get rid of a non-global declaration.
  *
@@ -94,11 +133,7 @@
 #define const
 #endif /* __STDC__ */
 
-#ifdef NO8BIT
-#define SET_SIZE	128
-#else
 #define SET_SIZE	256
-#endif
 
 #define MASKSIZE	(MASKBITS / MASKTYPE_WIDTH)
 
@@ -140,7 +175,7 @@ extern int	CHARTOBIT P ((int ch));
 #define CHARTOBIT(ch)	((ch) - 'A')
 # endif /* lint */
 #define LARGESTFLAG	26	/* 5 are needed for flagfield below */
-#define FLAGBASE	(MASKTYPE_WIDTH - 6)
+#define FLAGBASE	((MASKTYPE_WIDTH) - 6)
 #else
 # if MASKBITS <= 64
 #  ifndef lint
@@ -148,7 +183,7 @@ extern int	CHARTOBIT P ((int ch));
 #define CHARTOBIT(ch)	((ch) - 'A')
 #  endif /* lint */
 #define LARGESTFLAG	(64 - 6) /* 5 are needed for flagfield below */
-#define FLAGBASE	(MASKTYPE_WIDTH - 6)
+#define FLAGBASE	((MASKTYPE_WIDTH) - 6)
 # else
 #  ifndef lint
 #define BITTOCHAR(bit)	(bit)
@@ -171,10 +206,10 @@ extern int	CHARTOBIT P ((int ch));
 
 #ifdef ICHAR_IS_CHAR
 typedef unsigned char	ichar_t;	/* Internal character */
-#define icharlen(s)	strlen ((char *) s)
-#define icharcpy(a, b)	strcpy ((char *) a, (char *) b)
-#define icharcmp(a, b)	strcmp ((char *) a, (char *) b)
-#define icharncmp(a, b, n) strncmp ((char *) a, (char *) b, n)
+#define icharlen(s)	strlen ((char *) (s))
+#define icharcpy(a, b)	strcpy ((char *) (a), (char *) (b))
+#define icharcmp(a, b)	strcmp ((char *) (a), (char *) (b))
+#define icharncmp(a, b, n) strncmp ((char *) (a), (char *) (b), (n))
 #define chartoichar(x)	((ichar_t) (x))
 #else
 typedef unsigned short	ichar_t;	/* Internal character */
@@ -184,7 +219,7 @@ typedef unsigned short	ichar_t;	/* Internal character */
 struct dent
     {
     struct dent *	next;
-    char *		word;
+    unsigned char *	word;
     MASKTYPE		mask[MASKSIZE];
 #ifdef FULLMASKSET
     char		flags;
@@ -195,7 +230,6 @@ struct dent
 ** Flags in the directory entry.  If FULLMASKSET is undefined, these are
 ** stored in the highest bits of the last longword of the mask field.  If
 ** FULLMASKSET is defined, they are stored in the extra "flags" field.
-#ifndef NO_CAPITALIZATION_SUPPORT
 **
 ** If a word has only one capitalization form, and that form is not
 ** FOLLOWCASE, it will have exactly one entry in the dictionary.  The
@@ -255,7 +289,6 @@ struct dent
 ** (KEEP flag set) and the user adds "alpha" with the KEEP flag clear, a
 ** multiple-variant entry will be created so that "alpha" will be accepted
 ** but only "ALPHA" will actually be kept.
-#endif
 */
 #ifdef FULLMASKSET
 #define flagfield	flags
@@ -264,9 +297,6 @@ struct dent
 #endif
 #define USED		((MASKTYPE) 1 << (FLAGBASE + 0))
 #define KEEP		((MASKTYPE) 1 << (FLAGBASE + 1))
-#ifdef NO_CAPITALIZATION_SUPPORT
-#define ALLFLAGS	(USED | KEEP)
-#else /* NO_CAPITALIZATION_SUPPORT */
 #define ANYCASE		((MASKTYPE) 0 << (FLAGBASE + 2))
 #define ALLCAPS		((MASKTYPE) 1 << (FLAGBASE + 2))
 #define CAPITALIZED	((MASKTYPE) 2 << (FLAGBASE + 2))
@@ -275,7 +305,6 @@ struct dent
 #define MOREVARIANTS	((MASKTYPE) 1 << (FLAGBASE + 4))
 #define ALLFLAGS	(USED | KEEP | CAPTYPEMASK | MOREVARIANTS)
 #define captype(x)	((x) & CAPTYPEMASK)
-#endif /* NO_CAPITALIZATION_SUPPORT */
 
 /*
  * Language tables used to encode prefix and suffix information.
@@ -315,7 +344,7 @@ struct flagptr
  */
 struct strchartype
     {
-    char *		name;			/* Name of the type */
+    unsigned char *	name;			/* Name of the type */
     char *		deformatter;		/* Deformatter to use */
     char *		suffixes;		/* File suffixes, null seps */
     };
@@ -327,19 +356,19 @@ struct hashheader
     {
     unsigned short magic;    	    	    	/* Magic number for ID */
     unsigned short compileoptions;		/* How we were compiled */
-    short maxstringchars;			/* Max # strchrs we support */
-    short maxstringcharlen;			/* Max strchr len supported */
-    short compoundmin;				/* Min lth of compound parts */
+    unsigned short maxstringchars;		/* Max # strchrs we support */
+    unsigned short maxstringcharlen;		/* Max strchr len supported */
+    unsigned short compoundmin;			/* Min lth of compound parts */
     short compoundbit;				/* Flag 4 compounding roots */
-    int stringsize;				/* Size of string table */
-    int lstringsize;				/* Size of lang. str tbl */
-    int tblsize;				/* No. entries in hash tbl */
-    int stblsize;				/* No. entries in sfx tbl */
-    int ptblsize;				/* No. entries in pfx tbl */
-    int sortval;				/* Largest sort ID assigned */
-    int nstrchars;				/* No. strchars defined */
-    int nstrchartype;				/* No. strchar types */
-    int strtypestart;				/* Start of strtype table */
+    unsigned int stringsize;			/* Size of string table */
+    unsigned int lstringsize;			/* Size of lang. str tbl */
+    unsigned int tblsize;			/* No. entries in hash tbl */
+    unsigned int stblsize;			/* No. entries in sfx tbl */
+    unsigned int ptblsize;			/* No. entries in pfx tbl */
+    unsigned int sortval;			/* Largest sort ID assigned */
+    unsigned int nstrchars;			/* No. strchars defined */
+    unsigned int nstrchartype;			/* No. strchar types */
+    unsigned int strtypestart;			/* Start of strtype table */
     char nrchars[5];				/* Nroff special characters */
     char texchars[13];				/* TeX special characters */
     char compoundflag;				/* Compund-word handling */
@@ -353,7 +382,8 @@ struct hashheader
     char lowerchars[SET_SIZE + MAXSTRINGCHARS]; /* NZ for lowercase chars */
     char boundarychars[SET_SIZE + MAXSTRINGCHARS]; /* NZ for boundary chars */
     char stringstarts[SET_SIZE];		/* NZ if char can start str */
-    char stringchars[MAXSTRINGCHARS][MAXSTRINGCHARLEN + 1]; /* String chars */
+    unsigned char stringchars[MAXSTRINGCHARS][MAXSTRINGCHARLEN + 1];
+						/* String chars */
     unsigned int stringdups[MAXSTRINGCHARS];	/* No. of "base" char */
     int dupnos[MAXSTRINGCHARS];			/* Dup char ID # */
     unsigned short magic2;			/* Second magic for dbl chk */
@@ -363,16 +393,8 @@ struct hashheader
 #define MAGIC			0x9602
 
 /* compile options, put in the hash header for consistency checking */
-#ifdef NO8BIT
-# define MAGIC8BIT		0x01
-#else
-# define MAGIC8BIT		0x00
-#endif
-#ifdef NO_CAPITALIZATION_SUPPORT
-# define MAGICCAPITALIZATION	0x00
-#else
-# define MAGICCAPITALIZATION	0x02
-#endif
+# define MAGICNOTUSED1		0x01		/* No longer used */
+# define MAGICNOTUSED2		0x02		/* No longer used */
 #if MASKBITS <= 32
 # define MAGICMASKSET		0x00
 #else
@@ -387,7 +409,7 @@ struct hashheader
 # endif
 #endif
 
-#define COMPILEOPTIONS	(MAGIC8BIT | MAGICCAPITALIZATION | MAGICMASKSET)
+#define COMPILEOPTIONS	(MAGICNOTUSED1 | MAGICNOTUSED2 | MAGICMASKSET)
 
 /*
  * Structure used to record data about successful lookups; these values
@@ -400,6 +422,21 @@ struct success
     struct flagent *	suffix;		/* Suffix flag used, or NULL */
     };
 
+/*
+ * Structure used to describe keyword-lookup tables.  The lookup
+ * routine uses binary search on the keyword array.  Maxlen and minlen
+ * are just optimizations: if the string length isn't in this range,
+ * the lookup routine can fail immediately.
+ */
+struct kwtable
+    {
+    unsigned char **	kwlist;		/* Sorted array of keywords */
+    unsigned int	numkw;		/* Number of keywords in array */
+    unsigned int	minlen;		/* Length of shortest keyword */
+    unsigned int	maxlen;		/* Length of longest keyword */
+    int			forceupper;	/* NZ to force uppercase in match */
+    };
+
 /*
 ** Offsets into the nroff special-character array
 */
@@ -426,6 +463,13 @@ struct success
 #define TEXDOT		hashheader.texchars[11]
 #define TEXPERCENT	hashheader.texchars[12]
 
+#define HTMLTAGSTART	'<'
+#define HTMLTAGEND	'>'
+#define HTMLSLASH	'/'
+#define HTMLSPECSTART	'&'
+#define HTMLSPECEND   	';'
+#define HTMLQUOTE	'"'
+
 /*
 ** Values for compoundflag
 */
@@ -433,6 +477,14 @@ struct success
 #define COMPOUND_ANYTIME	1	/* Accept run-together words */
 #define COMPOUND_CONTROLLED	2	/* Compounds controlled by afx flags */
 
+/*
+** Values for deftflag and tflag (deformatting style)
+*/
+#define DEFORMAT_NONE		0	/* No deformatting (or external) */
+#define DEFORMAT_NROFF		1	/* Nroff/troff-style deformatting */
+#define DEFORMAT_TEX		2	/* TeX/LaTeX-style deformatting */
+#define DEFORMAT_SGML		3	/* SGML/HTML-style deformatting */
+
 /*
 ** The isXXXX macros normally only check ASCII range, and don't support
 ** the character sets of other languages.  These private versions handle
@@ -448,15 +500,15 @@ extern int	isstringstart P ((unsigned int ch));
 extern ichar_t	mytolower P ((unsigned int ch));
 extern ichar_t	mytoupper P ((unsigned int ch));
 #else /* lint */
-#define myupper(X)	(hashheader.upperchars[X])
-#define mylower(X)	(hashheader.lowerchars[X])
+#define myupper(X)	(hashheader.upperchars[(X)])
+#define mylower(X)	(hashheader.lowerchars[(X)])
 #define myspace(X)	(((X) > 0)  &&  ((X) < 0x80) \
 			  &&  isspace((unsigned char) (X)))
-#define iswordch(X)	(hashheader.wordchars[X])
-#define isboundarych(X) (hashheader.boundarychars[X])
+#define iswordch(X)	(hashheader.wordchars[(X)])
+#define isboundarych(X) (hashheader.boundarychars[(X)])
 #define isstringstart(X) (hashheader.stringstarts[(unsigned char) (X)])
-#define mytolower(X)	(hashheader.lowerconv[X])
-#define mytoupper(X)	(hashheader.upperconv[X])
+#define mytolower(X)	(hashheader.lowerconv[(X)])
+#define mytoupper(X)	(hashheader.upperconv[(X)])
 #endif /* lint */
 
 /*
@@ -473,15 +525,17 @@ extern ichar_t	mytoupper P ((unsigned int ch));
 ** returns false.  This macro is a great example of how NOT to write
 ** readable C.
 */
-#define isstringch(ptr, canon)	(isstringstart (*ptr) \
-				  &&  stringcharlen (ptr, canon) > 0)
+#define isstringch(ptr, canon)	(isstringstart (*(ptr)) \
+				  &&  stringcharlen ((ptr), (canon)) > 0)
 #define l_isstringch(ptr, len, canon)	\
-				(isstringstart (*ptr) \
-				  &&  (len = stringcharlen (ptr, canon)) > 0)
+				(isstringstart (*(ptr)) \
+				  &&  (len = stringcharlen ((ptr), (canon))) \
+				    > 0)
 #define l1_isstringch(ptr, len, canon)	\
 				(len = 1, \
-				  isstringstart (*ptr) \
-				    &&  ((len = stringcharlen (ptr, canon)) \
+				  isstringstart (*(ptr)) \
+				    &&  ((len = \
+					  stringcharlen ((ptr), (canon))) \
 					> 0 \
 				      ? 1 : (len = 1, 0)))
 
@@ -516,10 +570,15 @@ EXTERN int	li;	/* lines */
 EXTERN int	co;	/* columns */
 
 EXTERN int	contextsize;	/* number of lines of context to show */
-EXTERN char	contextbufs[MAXCONTEXT][BUFSIZ]; /* Context of current line */
+EXTERN unsigned char
+		contextbufs[MAXCONTEXT][BUFSIZ]; /* Context of current line */
 EXTERN int	contextoffset;	/* Offset of line start in contextbufs[0] */
-EXTERN char *	currentchar;	/* Location in contextbufs */
-EXTERN char	ctoken[INPUTWORDLEN + MAXAFFIXLEN]; /* Current token as char */
+EXTERN unsigned char *
+		currentchar;	/* Location in contextbufs */
+EXTERN unsigned char
+		ctoken[INPUTWORDLEN + MAXAFFIXLEN]; /* Current token as char */
+EXTERN unsigned char
+		filteredbuf[BUFSIZ]; /* Filtered line */
 EXTERN ichar_t	itoken[INPUTWORDLEN + MAXAFFIXLEN]; /* Ctoken as ichar_t str */
 
 EXTERN char	termcap[2048];	/* termcap entry */
@@ -530,12 +589,14 @@ EXTERN int	numhits;	/* number of hits in dictionary lookups */
 EXTERN struct success
 		hits[MAX_HITS]; /* table of hits gotten in lookup */
 
-EXTERN char *	hashstrings;	/* Strings in hash table */
+EXTERN unsigned char *
+		hashstrings;	/* Strings in hash table */
 EXTERN struct hashheader
 		hashheader;	/* Header of hash table */
 EXTERN struct dent *
 		hashtbl;	/* Main hash table, for dictionary */
-EXTERN int	hashsize;	/* Size of main hash table */
+EXTERN unsigned int
+		hashsize;	/* Size of main hash table */
 
 EXTERN char	hashname[MAXPATHLEN]; /* Name of hash table file */
 
@@ -551,8 +612,10 @@ EXTERN int	ukillchar;	/* User's kill character */
 EXTERN unsigned int laststringch; /* Number of last string character */
 EXTERN int	defdupchar;	/* Default duplicate string type */
 
-EXTERN int	numpflags;		/* Number of prefix flags in table */
-EXTERN int	numsflags;		/* Number of suffix flags in table */
+EXTERN unsigned int
+		numpflags;		/* Number of prefix flags in table */
+EXTERN unsigned int
+		numsflags;		/* Number of suffix flags in table */
 EXTERN struct flagptr pflagindex[SET_SIZE + MAXSTRINGCHARS];
 					/* Fast index to pflaglist */
 EXTERN struct flagent *	pflaglist;	/* Prefix flag control list */
@@ -565,6 +628,7 @@ EXTERN struct strchartype *		/* String character type collection */
 
 EXTERN FILE *	infile;			/* File being corrected */
 EXTERN FILE *	outfile;		/* Corrected copy of infile */
+EXTERN FILE*	sourcefile;		/* File with full input data */
 
 EXTERN char *	askfilename;		/* File specified in -f option */
 
@@ -588,6 +652,14 @@ EXTERN int	easypossibilities; /* Number of "easy" corrections found */
 EXTERN int	Trynum;		/* Size of "Try" array */
 EXTERN ichar_t	Try[SET_SIZE + MAXSTRINGCHARS];
 
+/*
+ * Tables of keywords, used in defmt.c
+ */
+EXTERN struct kwtable htmlchecklist;	/* List of HTML tags to always check */
+EXTERN struct kwtable htmlignorelist;	/* List of HTML tags to be ignored */
+EXTERN struct kwtable texskip1list;	/* List of TEX tags to skip 1 arg */
+EXTERN struct kwtable texskip2list;	/* List of TEX tags to skip 2 args */
+
 /*
  * Initialized variables.  These are generated using macros so that they
  * may be consistently declared in all programs.  Numerous examples of
@@ -605,7 +677,9 @@ INIT (int minimenusize, 2);		/* MUST be either 2 or zero */
 INIT (int minimenusize, 0);		/* MUST be either 2 or zero */
 #endif /* MINIMENU */
 
+INIT (int askverbose, 0);		/* NZ for verbose ask mode */
 INIT (int eflag, 0);			/* NZ for expand mode */
+INIT (char * defmtpgm, NULL);		/* Filter to use for deformatting */
 INIT (int dumpflag, 0);			/* NZ to do dump mode */
 INIT (int fflag, 0);			/* NZ if -f specified */
 #ifndef USG
@@ -613,11 +687,26 @@ INIT (int sflag, 0);			/* NZ to stop self after EOF */
 #endif
 INIT (int vflag, 0);			/* NZ to display characters as M-xxx */
 INIT (int xflag, DEFNOBACKUPFLAG);	/* NZ to suppress backups */
-INIT (int deftflag, -1);		/* NZ for TeX mode by default */
-INIT (int tflag, DEFTEXFLAG);		/* NZ for TeX mode in current file */
+INIT (int deftflag, -1);		/* Default deformatting mode, chosen */
+					/* ..from DEFORMAT_* values */
+INIT (int tflag, DEFTEXFLAG);		/* Deformatting for current file */
 INIT (int prefstringchar, -1);		/* Preferred string character type */
 
+INIT (int insidehtml, 0);		/* Flag to indicate we're amid HTML */
+					/*   0 = normal text */
+#define HTML_IN_TAG	0x01		/*   in <...> tag */
+#define HTML_IN_ENDTAG	0x02		/*   in </...> tag */
+#define HTML_IN_SPEC	0x04		/*   in &...; special-char sequence */
+#define HTML_IN_QUOTE	0x08		/*   in quoted text within tag */
+#define HTML_CHECKING_QUOTE 0x10	/*   checking quoted text within tag */
+#define HTML_IGNORE	0x20		/*   in text to be ignored */
+					/*   ...must be last, because we use */
+					/*   ...the higher bits as a nesting */
+					/*   ...counter */
+#define HTML_ISIGNORED	(~(HTML_IGNORE - 1)) /* Mask for testing ignore bits */
+
 INIT (int terse, 0);			/* NZ for "terse" mode */
+INIT (int correct_verbose_mode, 0);	/* NZ for "verbose" -a mode */
 
 INIT (char tempfile[MAXPATHLEN], "");	/* Name of file we're spelling into */
 
