@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: lookup.c,v 1.42 1995/01/08 23:23:42 geoff Exp $";
+    "$Id: lookup.c,v 1.47 2000/08/22 10:52:25 geoff Exp $";
 #endif
 
 /*
@@ -8,7 +8,7 @@ static char Rcs_Id[] =
  *
  * Pace Willisson, 1983
  *
- * Copyright 1987, 1988, 1989, 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * Copyright 1987, 1988, 1989, 1992, 1993, 1999, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,6 +47,22 @@ static char Rcs_Id[] =
 
 /*
  * $Log: lookup.c,v $
+ * Revision 1.47  2000/08/22 10:52:25  geoff
+ * Fix a whole bunch of signed/unsigned discrepancies.
+ *
+ * Revision 1.46  1999/01/18 03:28:35  geoff
+ * Turn some char declarations into unsigned char, so that we won't have
+ * sign-extension problems.
+ *
+ * Revision 1.45  1999/01/07  01:22:51  geoff
+ * Update the copyright.
+ *
+ * Revision 1.44  1998/07/06  05:44:47  geoff
+ * Use off_t in lseeks, for 64-bit and NetBSD machines
+ *
+ * Revision 1.43  1997/12/02  06:24:56  geoff
+ * Get rid of some compile options that really shouldn't be optional.
+ *
  * Revision 1.42  1995/01/08  23:23:42  geoff
  * Support MSDOS_BINARY_OPEN when opening the hash file to read it in.
  *
@@ -66,7 +82,7 @@ static void	dumpindex P ((struct flagptr * indexp, int depth));
 #endif /* INDEXDUMP */
 struct dent *	lookup P ((ichar_t * word, int dotree));
 
-static		inited = 0;
+static int	inited = 0;
 
 int linit ()
     {
@@ -91,7 +107,7 @@ int linit ()
     hashsize = read (hashfd, (char *) &hashheader, sizeof hashheader);
     if (hashsize < sizeof hashheader)
 	{
-	if (hashsize < 0)
+	if (hashsize == (unsigned int) -1)
 	    (void) fprintf (stderr, LOOKUP_C_CANT_READ, hashname);
 	else if (hashsize == 0)
 	    (void) fprintf (stderr, LOOKUP_C_NULL_HASH, hashname);
@@ -145,7 +161,8 @@ int linit ()
 	hashtbl[0].next = NULL;
 	hashtbl[0].flagfield &= ~(USED | KEEP);
 	/* The flag bits don't matter, but calloc cleared them. */
-	hashstrings = (char *) malloc ((unsigned) hashheader.lstringsize);
+	hashstrings =
+	  (unsigned char *) malloc ((unsigned) hashheader.lstringsize);
 	}
     else
 	{
@@ -153,7 +170,8 @@ int linit ()
 	 (struct dent *)
 	    malloc ((unsigned) hashheader.tblsize * sizeof (struct dent));
 	hashsize = hashheader.tblsize;
-	hashstrings = (char *) malloc ((unsigned) hashheader.stringsize);
+	hashstrings =
+	  (unsigned char *) malloc ((unsigned) hashheader.stringsize);
 	}
     numsflags = hashheader.stblsize;
     numpflags = hashheader.ptblsize;
@@ -174,29 +192,29 @@ int linit ()
 	 * hash table.
 	 */
 	if (read (hashfd, hashstrings, (unsigned) hashheader.lstringsize)
-	  != hashheader.lstringsize)
+	  != (int) hashheader.lstringsize)
 	    {
 	    (void) fprintf (stderr, LOOKUP_C_BAD_FORMAT);
 	    return (-1);
 	    }
 	(void) lseek (hashfd,
-	  (long) hashheader.stringsize - (long) hashheader.lstringsize
-	    + (long) hashheader.tblsize * (long) sizeof (struct dent),
+	  (off_t) hashheader.stringsize - (off_t) hashheader.lstringsize
+	    + (off_t) hashheader.tblsize * (off_t) sizeof (struct dent),
 	  1);
 	}
     else
 	{
 	if (read (hashfd, hashstrings, (unsigned) hashheader.stringsize)
-	    != hashheader.stringsize
+	    != (int) hashheader.stringsize
 	  ||  read (hashfd, (char *) hashtbl,
 	      (unsigned) hashheader.tblsize * sizeof (struct dent))
-	    != hashheader.tblsize * sizeof (struct dent))
+	    != (int) (hashheader.tblsize * sizeof (struct dent)))
 	    {
 	    (void) fprintf (stderr, LOOKUP_C_BAD_FORMAT);
 	    return (-1);
 	    }
 	}
-    if (read (hashfd, (char *) sflaglist,
+    if ((unsigned) read (hashfd, (char *) sflaglist,
 	(unsigned) (numsflags + numpflags) * sizeof (struct flagent))
       != (numsflags + numpflags) * sizeof (struct flagent))
 	{
@@ -209,7 +227,7 @@ int linit ()
 	{
 	for (i = hashsize, dp = hashtbl;  --i >= 0;  dp++)
 	    {
-	    if (dp->word == (char *) -1)
+	    if (dp->word == (unsigned char *) -1)
 		dp->word = NULL;
 	    else
 		dp->word = &hashstrings [ (int)(dp->word) ];
@@ -374,16 +392,16 @@ int linit ()
 	    return (-1);
 	    }
 	for (i = 0, nextchar = hashheader.strtypestart;
-	  i < hashheader.nstrchartype;
+	  i < (int) hashheader.nstrchartype;
 	  i++)
 	    {
 	    chartypes[i].name = &hashstrings[nextchar];
-	    nextchar += strlen (chartypes[i].name) + 1;
-	    chartypes[i].deformatter = &hashstrings[nextchar];
+	    nextchar += strlen ((char *) chartypes[i].name) + 1;
+	    chartypes[i].deformatter = (char *) &hashstrings[nextchar];
 	    nextchar += strlen (chartypes[i].deformatter) + 1;
-	    chartypes[i].suffixes = &hashstrings[nextchar];
+	    chartypes[i].suffixes = (char *) &hashstrings[nextchar];
 	    while (hashstrings[nextchar] != '\0')
-		nextchar += strlen (&hashstrings[nextchar]) + 1;
+		nextchar += strlen ((char *) &hashstrings[nextchar]) + 1;
 	    nextchar++;
 	    }
 	}
@@ -453,8 +471,8 @@ struct dent * lookup (s, dotree)
     int				dotree;
     {
     register struct dent *	dp;
-    register char *		s1;
-    char			schar[INPUTWORDLEN + MAXAFFIXLEN];
+    register unsigned char *	s1;
+    unsigned char		schar[INPUTWORDLEN + MAXAFFIXLEN];
 
     dp = &hashtbl[hash (s, hashsize)];
     if (ichartostr (schar, s, sizeof schar, 1))
@@ -463,12 +481,11 @@ struct dent * lookup (s, dotree)
 	{
 	/* quick strcmp, but only for equality */
 	s1 = dp->word;
-	if (s1  &&  s1[0] == schar[0]  &&  strcmp (s1 + 1, schar + 1) == 0)
+	if (s1  &&  s1[0] == schar[0]
+	  &&  strcmp ((char *) s1 + 1, schar + 1) == 0)
 	    return dp;
-#ifndef NO_CAPITALIZATION_SUPPORT
 	while (dp->flagfield & MOREVARIANTS)	/* Skip variations */
 	    dp = dp->next;
-#endif
 	}
     if (dotree)
 	{
