@@ -1,10 +1,10 @@
 #ifndef lint
 static char Rcs_Id[] =
-	"$Id: makedent.c,v 1.45 1994/12/27 23:08:52 geoff Exp $";
+	"$Id: makedent.c,v 1.52 2000/08/22 10:52:25 geoff Exp $";
 #endif
 
 /*
- * Copyright 1988, 1989, 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * Copyright 1988, 1989, 1992, 1993, 1999, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,6 +43,28 @@ static char Rcs_Id[] =
 
 /*
  * $Log: makedent.c,v $
+ * Revision 1.52  2000/08/22 10:52:25  geoff
+ * Fix a whole bunch of signed/unsigned discrepancies.
+ *
+ * Revision 1.51  1999/01/18 03:28:37  geoff
+ * Turn many char declarations into unsigned char, so that we won't have
+ * sign-extension problems.
+ *
+ * Revision 1.50  1999/01/07  01:23:09  geoff
+ * Update the copyright.
+ *
+ * Revision 1.49  1999/01/03  01:46:39  geoff
+ * Add support for the "plain" deformatters.
+ *
+ * Revision 1.48  1998/07/06  05:03:39  geoff
+ * Don't issue bad-flag messages in -a mode; it confuses emacs
+ *
+ * Revision 1.47  1997/12/02  06:24:58  geoff
+ * Get rid of some compile options that really shouldn't be optional.
+ *
+ * Revision 1.46  1995/11/08  05:09:20  geoff
+ * Allow sgml as a deformatter type.
+ *
  * Revision 1.45  1994/12/27  23:08:52  geoff
  * Add code to makedent to reject words that contain non-word characters.
  * This helps protect people who use ISO 8-bit characters when ispell
@@ -67,36 +89,33 @@ static char Rcs_Id[] =
 #include "proto.h"
 #include "msgs.h"
 
-int		makedent P ((char * lbuf, int lbuflen, struct dent * ent));
-#ifndef NO_CAPITALIZATION_SUPPORT
+int		makedent P ((unsigned char * lbuf, int lbuflen,
+		  struct dent * ent));
 long		whatcap P ((ichar_t * word));
-#endif
 int		addvheader P ((struct dent * ent));
 int		combinecaps P ((struct dent * hdr, struct dent * newent));
-#ifndef NO_CAPITALIZATION_SUPPORT
 static void	forcevheader P ((struct dent * hdrp, struct dent * oldp,
 		  struct dent * newp));
-#endif /* NO_CAPITALIZATION_SUPPORT */
 static int	combine_two_entries P ((struct dent * hdrp,
 		  struct dent * oldp, struct dent * newp));
 static int	acoversb P ((struct dent * enta, struct dent * entb));
 void		upcase P ((ichar_t * string));
 void		lowcase P ((ichar_t * string));
-void		chupcase P ((char * s));
+void		chupcase P ((unsigned char * s));
 static int	issubset P ((struct dent * ent1, struct dent * ent2));
 static void	combineaffixes P ((struct dent * ent1, struct dent * ent2));
 void		toutent P ((FILE * outfile, struct dent * hent,
 		  int onlykeep));
-static void	toutword P ((FILE * outfile, char * word,
+static void	toutword P ((FILE * outfile, unsigned char * word,
 		  struct dent * cent));
 static void	flagout P ((FILE * outfile, int flag));
-int		stringcharlen P ((char * bufp, int canonical));
-int		strtoichar P ((ichar_t * out, char * in, int outlen,
+int		stringcharlen P ((unsigned char * bufp, int canonical));
+int		strtoichar P ((ichar_t * out, unsigned char * in, int outlen,
 		  int canonical));
-int		ichartostr P ((char * out, ichar_t * in, int outlen,
+int		ichartostr P ((unsigned char * out, ichar_t * in, int outlen,
 		  int canonical));
-ichar_t *	strtosichar P ((char * in, int canonical));
-char *		ichartosstr P ((ichar_t * in, int canonical));
+ichar_t *	strtosichar P ((unsigned char * in, int canonical));
+unsigned char *	ichartosstr P ((ichar_t * in, int canonical));
 char *		printichar P ((int in));
 #ifndef ICHAR_IS_CHAR
 ichar_t *	icharcpy P ((ichar_t * out, ichar_t * in));
@@ -116,18 +135,18 @@ static int  	has_marker;
  */
 
 int makedent (lbuf, lbuflen, d)
-    char *		lbuf;
+    unsigned char *	lbuf;
     int			lbuflen;
     struct dent *	d;
     {
     ichar_t		ibuf[INPUTWORDLEN + MAXAFFIXLEN];
     ichar_t *		ip;
-    char *		p;
+    unsigned char *	p;
     int			bit;
-    int			len;
+    unsigned int	len;
 
     /* Strip off any trailing newline */
-    len = strlen (lbuf) - 1;
+    len = strlen ((char *) lbuf) - 1;
     if (lbuf[len] == '\n')
 	lbuf[len] = '\0';
 
@@ -138,7 +157,7 @@ int makedent (lbuf, lbuflen, d)
     d->flagfield |= USED;
     d->flagfield &= ~KEEP;
 
-    p = index (lbuf, hashheader.flagmarker);
+    p = (unsigned char *) index ((char *) lbuf, hashheader.flagmarker);
     if (p != NULL)
 	*p = 0;
 
@@ -149,7 +168,7 @@ int makedent (lbuf, lbuflen, d)
     if (strtoichar (ibuf, lbuf, INPUTWORDLEN * sizeof (ichar_t), 1)
       ||  ichartostr (lbuf, ibuf, lbuflen, 1))
 	{
-	(void) fprintf (stderr, WORD_TOO_LONG (lbuf));
+	(void) fprintf (stderr, WORD_TOO_LONG ((char *) lbuf));
 	return (-1);
 	}
     /*
@@ -163,40 +182,35 @@ int makedent (lbuf, lbuflen, d)
 	    if (!isboundarych (*ip)
 	      ||  ip == ibuf  ||  ip[1] == 0)
 		{
-		(void) fprintf (stderr, MAKEDENT_C_BAD_WORD_CHAR, lbuf);
+		(void) fprintf (stderr, MAKEDENT_C_BAD_WORD_CHAR,
+		  (char *) lbuf);
 		return -1;
 		}
 	    }
 	}
-    len = strlen (lbuf);
-#ifndef NO_CAPITALIZATION_SUPPORT
+    len = strlen ((char *) lbuf);
     /*
     ** Figure out the capitalization rules from the capitalization of
     ** the sample entry.
     */
     d->flagfield |= whatcap (ibuf);
-#endif
 
     if (len > INPUTWORDLEN - 1)
 	{
-	(void) fprintf (stderr, WORD_TOO_LONG (lbuf));
+	(void) fprintf (stderr, WORD_TOO_LONG ((char *) lbuf));
 	return (-1);
 	}
 
     d->word = mymalloc ((unsigned) len + 1);
     if (d->word == NULL)
 	{
-	(void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE, lbuf);
+	(void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE, (char *) lbuf);
 	return -1;
 	}
 
-    (void) strcpy (d->word, lbuf);
-#ifdef NO_CAPITALIZATION_SUPPORT
-    chupcase (d->word);
-#else /* NO_CAPITALIZATION_SUPPORT */
+    (void) strcpy ((char *) d->word, (char *) lbuf);
     if (captype (d->flagfield) != FOLLOWCASE)
 	chupcase (d->word);
-#endif /* NO_CAPITALIZATION_SUPPORT */
     if (p == NULL)
 	return (0);
 
@@ -206,7 +220,7 @@ int makedent (lbuf, lbuflen, d)
 	bit = CHARTOBIT ((unsigned char) *p);
 	if (bit >= 0  &&  bit <= LARGESTFLAG)
 	    SETMASKBIT (d->mask, bit);
-	else
+	else if (!aflag)
 	    (void) fprintf (stderr, BAD_FLAG, (unsigned char) *p);
 	p++;
 	if (*p == hashheader.flagmarker)
@@ -215,7 +229,6 @@ int makedent (lbuf, lbuflen, d)
     return (0);
     }
 
-#ifndef NO_CAPITALIZATION_SUPPORT
 /*
 ** Classify the capitalization of a sample entry.  Returns one of the
 ** four capitalization codes ANYCASE, ALLCAPS, CAPITALIZED, or FOLLOWCASE.
@@ -284,7 +297,7 @@ int addvheader (dp)
     tdent = (struct dent *) mymalloc (sizeof (struct dent));
     if (tdent == NULL)
 	{
-	(void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE, dp->word);
+	(void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE, (char *) dp->word);
 	return -1;
 	}
     *tdent = *dp;
@@ -293,14 +306,16 @@ int addvheader (dp)
     else
 	{
 	/* Followcase words need a copy of the capitalization */
-	tdent->word = mymalloc ((unsigned int) strlen (tdent->word) + 1);
+	tdent->word =
+	  mymalloc ((unsigned int) strlen ((char *) tdent->word) + 1);
 	if (tdent->word == NULL)
 	    {
-	    (void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE, dp->word);
+	    (void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE,
+	      (char *) dp->word);
 	    myfree ((char *) tdent);
 	    return -1;
 	    }
-	(void) strcpy (tdent->word, dp->word);
+	(void) strcpy ((char *) tdent->word, (char *) dp->word);
 	}
     chupcase (dp->word);
     dp->next = tdent;
@@ -308,7 +323,6 @@ int addvheader (dp)
     dp->flagfield |= (ALLCAPS | MOREVARIANTS);
     return 0;
     }
-#endif /* NO_CAPITALIZATION_SUPPORT */
 
 /*
 ** Combine and resolve the entries describing two capitalizations of the same
@@ -336,7 +350,6 @@ int addvheader (dp)
 **	(1) Add newp's affixes and KEEP flag to oldp, and discard newp.
 **	(2) Add oldp's affixes and KEEP flag to newp, replace oldp with
 **	    newp, and discard newp.
-#ifndef NO_CAPITALIZATION_SUPPORT
 **	(3) Insert newp as a new entry in the variants list.  If there is
 **	    currently no variant header, this requires adding one.  Adding a
 **	    header splits into two sub-cases:
@@ -349,7 +362,6 @@ int addvheader (dp)
 **
 **	    After newp has been added as a variant, its affixes and KEEP
 **	    flag are OR-ed into the variant header.
-#endif
 **
 ** So how to choose which?  The default is always case (3), which adds newp
 ** as a new entry in the variants list.  Cases (1) and (2) are symmetrical
@@ -367,13 +379,11 @@ int addvheader (dp)
 **	    (4c) If the words are FOLLOWCASE, the capitalizations match
 **		exactly.
 **
-#ifndef NO_CAPITALIZATION_SUPPORT
 **	(5) For entries with mismatched capitalization types, A covers B
 **	    if (4a) and (4b) are true, and:
 **
 **	    (5a) B is ALLCAPS, or
 **	    (5b) A is ANYCASE, and B is CAPITALIZED.
-#endif
 **
 ** For any "hdrp" without variants, oldp is the same as hdrp.  Otherwise,
 ** the above tests are applied using each variant in turn for oldp.
@@ -385,10 +395,8 @@ int combinecaps (hdrp, newp)
     {
     register struct dent *
 			oldp;	/* Current "oldp" entry */
-#ifndef NO_CAPITALIZATION_SUPPORT
     register struct dent *
 			tdent; /* Entry we'll add to the dictionary */
-#endif /* NO_CAPITALIZATION_SUPPORT */
     register int	retval = 0; /* Return value from combine_two_entries */
 
     /*
@@ -397,9 +405,6 @@ int combinecaps (hdrp, newp)
     ** it will return zero.
     */
     oldp = hdrp;
-#ifdef NO_CAPITALIZATION_SUPPORT
-    retval = combine_two_entries (hdrp, oldp, newp);
-#else /* NO_CAPITALIZATION_SUPPORT */
     if ((oldp->flagfield & (CAPTYPEMASK | MOREVARIANTS))
       == (ALLCAPS | MOREVARIANTS))
 	{
@@ -424,7 +429,8 @@ int combinecaps (hdrp, newp)
 	tdent = (struct dent *) mymalloc (sizeof (struct dent));
 	if (tdent == NULL)
 	    {
-	    (void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE, newp->word);
+	    (void) fprintf (stderr, MAKEDENT_C_NO_WORD_SPACE,
+	      (char *) newp->word);
 	    return -1;
 	    }
 	*tdent = *newp;
@@ -442,11 +448,9 @@ int combinecaps (hdrp, newp)
 	    myfree (newp->word);		/* newp->word isn't needed */
 	    }
 	}
-#endif /* NO_CAPITALIZATION_SUPPORT */
     return retval;
     }
 
-#ifndef NO_CAPITALIZATION_SUPPORT
 /*
 ** The following routine implements steps 3a and 3b in the commentary
 ** for "combinecaps".
@@ -464,7 +468,6 @@ static void forcevheader (hdrp, oldp, newp)
       != (ALLCAPS | MOREVARIANTS))
 	(void) addvheader (hdrp);
     }
-#endif /* NO_CAPITALIZATION_SUPPORT */
 
 /*
 ** This routine implements steps 4 and 5 of the commentary for "combinecaps".
@@ -495,11 +498,7 @@ static int combine_two_entries (hdrp, oldp, newp)
 	** the keep flag.
 	*/
 	combineaffixes (newp, oldp);
-#ifdef NO_CAPITALIZATION_SUPPORT
-	newp->flagfield |= (oldp->flagfield & KEEP);
-#else /* NO_CAPITALIZATION_SUPPORT */
 	newp->flagfield |= (oldp->flagfield & (KEEP | MOREVARIANTS));
-#endif /* NO_CAPITALIZATION_SUPPORT */
 	hdrp->flagfield |= (newp->flagfield & KEEP);
 	newp->next = oldp->next;
 	/*
@@ -509,17 +508,15 @@ static int combine_two_entries (hdrp, oldp, newp)
 	** that both words are the same length.
 	*/
 	if (oldp->word != NULL)
-	    (void) strcpy (oldp->word, newp->word);
+	    (void) strcpy ((char *) oldp->word, (char *) newp->word);
 	myfree (newp->word);	/* No longer needed */
 	newp->word = oldp->word;
 	*oldp = *newp;
-#ifndef NO_CAPITALIZATION_SUPPORT
 	/* We may need to add a header if newp is followcase */
 	if (captype (newp->flagfield) == FOLLOWCASE
 	  &&  (hdrp->flagfield & (CAPTYPEMASK | MOREVARIANTS))
 	    != (ALLCAPS | MOREVARIANTS))
 	    (void) addvheader (hdrp);
-#endif /* NO_CAPITALIZATION_SUPPORT */
 	return 1;
 	}
     else
@@ -551,17 +548,10 @@ static int acoversb (enta, entb)
 	}
 
     /* Rules (4a) and (4b) are satisfied;  check for capitalization match */
-#ifdef NO_CAPITALIZATION_SUPPORT
-#ifdef lint
-    return subset;				/* Just so it gets used */
-#else /* lint */
-    return 1;					/* All words match */
-#endif /* lint */
-#else /* NO_CAPITALIZATION_SUPPORT */
     if (((enta->flagfield ^ entb->flagfield) & CAPTYPEMASK) == 0)
 	{
 	if (captype (enta->flagfield) != FOLLOWCASE	/* Condition (4c) */
-	  ||  strcmp (enta->word, entb->word) == 0)
+	  ||  strcmp ((char *) enta->word, (char *) entb->word) == 0)
 	    return 1;				/* Perfect match */
 	else
 	    return 0;
@@ -575,7 +565,6 @@ static int acoversb (enta, entb)
 	return 1;
     else
 	return 0;
-#endif /* NO_CAPITALIZATION_SUPPORT */
     }
 
 void upcase (s)
@@ -605,13 +594,13 @@ void lowcase (s)
  * slower than the normal upcase.  The input must be in canonical form.
  */
 void chupcase (s)
-    char *	s;
+    unsigned char *	s;
     {
-    ichar_t *	is;
+    ichar_t *		is;
 
     is = strtosichar (s, 1);
     upcase (is);
-    (void) ichartostr (s, is, strlen (s) + 1, 1);
+    (void) ichartostr (s, is, strlen ((char *) s) + 1, 1);
     }
 
 /*
@@ -682,16 +671,12 @@ void toutent (toutfile, hent, onlykeep)
     struct dent *	hent;
     register int	onlykeep;
     {
-#ifdef NO_CAPITALIZATION_SUPPORT
-    if (!onlykeep  ||  (hent->flagfield & KEEP))
-	toutword (toutfile, hent->word, hent);
-#else
     register struct dent * cent;
     ichar_t		wbuf[INPUTWORDLEN + MAXAFFIXLEN];
 
     cent = hent;
     if (strtoichar (wbuf, cent->word, INPUTWORDLEN, 1))
-	(void) fprintf (stderr, WORD_TOO_LONG (cent->word));
+	(void) fprintf (stderr, WORD_TOO_LONG ((char *) cent->word));
     for (  ;  ;  )
 	{
 	if (!onlykeep  ||  (cent->flagfield & KEEP))
@@ -725,18 +710,17 @@ void toutent (toutfile, hent, onlykeep)
 	else
 	    break;
 	}
-#endif
     }
 		
 static void toutword (toutfile, word, cent)
     register FILE *	toutfile;
-    char *		word;
+    unsigned char *	word;
     register struct dent * cent;
     {
     register int	bit;
 
     has_marker = 0;
-    (void) fprintf (toutfile, "%s", word);
+    (void) fprintf (toutfile, "%s", (char *) word);
     for (bit = 0;  bit < LARGESTFLAG;  bit++)
 	{
 	if (TSTMASKBIT (cent->mask, bit))
@@ -766,15 +750,18 @@ static void flagout (toutfile, flag)
  * as case conversion.
  */
 int stringcharlen (bufp, canonical)
-    char *		bufp;
+    unsigned char *	bufp;
     int			canonical;	/* NZ if input is in canonical form */
     {
 #ifdef SLOWMULTIPLY
-    static char *	sp[MAXSTRINGCHARS];
+    static unsigned char *
+			sp[MAXSTRINGCHARS];
     static int		inited = 0;
 #endif /* SLOWMULTIPLY */
-    register char *	bufcur;
-    register char *	stringcur;
+    register unsigned char *
+			bufcur;
+    register unsigned char *
+			stringcur;
     register int	stringno;
     register int	lowstringno;
     register int	highstringno;
@@ -802,11 +789,7 @@ int stringcharlen (bufp, canonical)
 	bufcur = bufp;
 	while (*stringcur)
 	    {
-#ifdef NO8BIT
-	    if (((*bufcur++ ^ *stringcur) & 0x7F) != 0)
-#else /* NO8BIT */
 	    if (*bufcur++ != *stringcur)
-#endif /* NO8BIT */
 		break;
 	    /*
 	    ** We can't use autoincrement above because of the
@@ -830,17 +813,10 @@ int stringcharlen (bufp, canonical)
 		--stringcur;
 	    }
 	/* No match - choose which side to search on */
-#ifdef NO8BIT
-	if ((*--bufcur & 0x7F) < (*stringcur & 0x7F))
-	    highstringno = stringno - 1;
-	else if ((*bufcur & 0x7F) > (*stringcur & 0x7F))
-	    lowstringno = stringno + 1;
-#else /* NO8BIT */
 	if (*--bufcur < *stringcur)
 	    highstringno = stringno - 1;
 	else if (*bufcur > *stringcur)
 	    lowstringno = stringno + 1;
-#endif /* NO8BIT */
 	else if (dupwanted < hashheader.dupnos[stringno])
 	    highstringno = stringno - 1;
 	else
@@ -851,14 +827,14 @@ int stringcharlen (bufp, canonical)
     }
 
 /*
- * Convert an external string to an ichar_t string.  If necessary, the parity
- * bit is stripped off as part of the process.
+ * Convert an external string to an ichar_t string.
  *
  * Returns NZ if the output string overflowed.
  */
 int strtoichar (out, in, outlen, canonical)
     register ichar_t *	out;		/* Where to put result */
-    register char *	in;		/* String to convert */
+    register unsigned char *
+			in;		/* String to convert */
     int			outlen;		/* Size of output buffer, *BYTES* */
     int			canonical;	/* NZ if input is in canonical form */
     {
@@ -870,7 +846,7 @@ int strtoichar (out, in, outlen, canonical)
 	if (l1_isstringch (in, len, canonical))
 	    *out++ = SET_SIZE + laststringch;
 	else
-	    *out++ = *in & NOPARITY;
+	    *out++ = *in;
 	}
     *out = 0;
     return outlen <= 0;
@@ -887,16 +863,19 @@ int strtoichar (out, in, outlen, canonical)
  * Returns NZ if the output string overflowed.
  */
 int ichartostr (out, in, outlen, canonical)
-    register char *	out;		/* Where to put result */
+    register unsigned char *
+			out;		/* Where to put result */
     register ichar_t *	in;		/* String to convert */
     int			outlen;		/* Size of output buffer, bytes */
     int			canonical;	/* NZ for canonical form */
     {
-    register int	ch;		/* Next character to store */
+    register unsigned int
+			ch;		/* Next character to store */
     register int	i;		/* Index into duplicates list */
-    register char *	scharp;		/* Pointer into a string char */
+    register unsigned char *
+			scharp;		/* Pointer into a string char */
 
-    while (--outlen > 0  &&  (ch = *in++) != 0)
+    while (--outlen > 0  &&  (ch = *in++) != '\0')
 	{
 	if (ch < SET_SIZE)
 	    *out++ = (char) ch;
@@ -929,24 +908,25 @@ int ichartostr (out, in, outlen, canonical)
  * Convert a string to an ichar_t, storing the result in a static area.
  */
 ichar_t * strtosichar (in, canonical)
-    char *		in;		/* String to convert */
+    unsigned char *	in;		/* String to convert */
     int			canonical;	/* NZ if input is in canonical form */
     {
     static ichar_t	out[STRTOSICHAR_SIZE / sizeof (ichar_t)];
 
     if (strtoichar (out, in, sizeof out, canonical))
-	(void) fprintf (stderr, WORD_TOO_LONG (in));
+	(void) fprintf (stderr, WORD_TOO_LONG ((char *) in));
     return out;
     }
 
 /*
  * Convert an ichar_t to a string, storing the result in a static area.
  */
-char * ichartosstr (in, canonical)
+unsigned char * ichartosstr (in, canonical)
     ichar_t *		in;		/* Internal string to convert */
     int			canonical;	/* NZ for canonical conversion */
     {
-    static char		out[ICHARTOSSTR_SIZE];
+    static unsigned char
+			out[ICHARTOSSTR_SIZE];
 
     if (ichartostr (out, in, sizeof out, canonical))
 	(void) fprintf (stderr, WORD_TOO_LONG (out));
@@ -968,7 +948,8 @@ char * printichar (in)
 	out[1] = '\0';
 	}
     else
-	(void) strcpy (out, hashheader.stringchars[(unsigned) in - SET_SIZE]);
+	(void) strcpy (out,
+	  (char *) hashheader.stringchars[(unsigned) in - SET_SIZE]);
     return out;
     }
 
@@ -1046,24 +1027,28 @@ int findfiletype (name, searchnames, deformatter)
     {
     char *		cp;		/* Pointer into suffix list */
     int			cplen;		/* Length of current suffix */
-    register int	i;		/* Index into type table */
+    register unsigned int
+			i;		/* Index into type table */
     int			len;		/* Length of the name */
 
-    /*
-     * Note:  for now, the deformatter is set to 1 for tex, 0 for nroff.
-     * Further, we assume that it's one or the other, so that a test
-     * for tex is sufficient.  This needs to be generalized.
-     */
     len = strlen (name);
     if (searchnames)
 	{
 	for (i = 0;  i < hashheader.nstrchartype;  i++)
 	    {
-	    if (strcmp (name, chartypes[i].name) == 0)
+	    if (strcmp (name, (char *) chartypes[i].name) == 0)
 		{
 		if (deformatter != NULL)
-		    *deformatter =
-		      (strcmp (chartypes[i].deformatter, "tex") == 0);
+		    {
+		    if (strcmp (chartypes[i].deformatter, "plain") == 0)
+			*deformatter = DEFORMAT_NONE;
+		    else if (strcmp (chartypes[i].deformatter, "tex") == 0)
+			*deformatter = DEFORMAT_TEX;
+		    else if (strcmp (chartypes[i].deformatter, "sgml") == 0)
+			*deformatter = DEFORMAT_SGML;
+		    else
+			*deformatter = DEFORMAT_NROFF;
+		    }
 		return i;
 		}
 	    }
@@ -1076,8 +1061,14 @@ int findfiletype (name, searchnames, deformatter)
 	    if (len >= cplen  &&  strcmp (&name[len - cplen], cp) == 0)
 		{
 		if (deformatter != NULL)
-		    *deformatter =
-		      (strcmp (chartypes[i].deformatter, "tex") == 0);
+		    {
+		    if (strcmp (chartypes[i].deformatter, "tex") == 0)
+			*deformatter = DEFORMAT_TEX;
+		    else if (strcmp (chartypes[i].deformatter, "sgml") == 0)
+			*deformatter = DEFORMAT_SGML;
+		    else
+			*deformatter = DEFORMAT_NROFF;
+		    }
 		return i;
 		}
 	    }
