@@ -1,10 +1,10 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: tgood.c,v 1.32 1994/11/02 06:56:16 geoff Exp $";
+    "$Id: tgood.c,v 1.39 2000/11/14 07:27:04 geoff Exp $";
 #endif
 
 /*
- * Copyright 1987, 1988, 1989, 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * Copyright 1987-1989, 1992, 1993, 1999, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,6 +49,36 @@ static char Rcs_Id[] =
 
 /*
  * $Log: tgood.c,v $
+ * Revision 1.39  2000/11/14 07:27:04  geoff
+ * Use messages from msgs.h for the "too many expansions" errors.
+ *
+ * Revision 1.38  2000/08/22 10:52:25  geoff
+ * Fix a whole bunch of signed/unsigned discrepancies.
+ *
+ * Revision 1.37  1999/01/18 03:28:43  geoff
+ * Turn many char declarations into unsigned char, so that we won't have
+ * sign-extension problems.
+ *
+ * Revision 1.36  1999/01/07  01:57:43  geoff
+ * Update the copyright.
+ *
+ * Revision 1.35  1997/12/21  23:10:24  geoff
+ * Fix the generation of expansions so that only one unique expansion is
+ * produced for any given root/affix combination.  For example, if both
+ * the A and the N flags can add "n" to the end of a word (as in the
+ * deutsch.aff file), expanding foo/AN will produce "foon" only once.
+ * This has the side effect that the "-e4" option will generate a more
+ * accurate ratio for flag productivity, which in turn means that
+ * munchlist will do a better job of choosing an optimal set of roots and
+ * flags.
+ *
+ * Revision 1.34  1997/12/03  05:28:13  geoff
+ * Fix a bug that caused suffix expansions to be printed with the wrong
+ * capitalization when all but one character of the root was stripped.
+ *
+ * Revision 1.33  1997/12/02  06:25:09  geoff
+ * Get rid of some compile options that really shouldn't be optional.
+ *
  * Revision 1.32  1994/11/02  06:56:16  geoff
  * Remove the anyword feature, which I've decided is a bad idea.
  *
@@ -73,6 +103,7 @@ static char Rcs_Id[] =
 #include <ctype.h>
 #include "config.h"
 #include "ispell.h"
+#include "msgs.h"
 #include "proto.h"
 
 void		chk_aff P ((ichar_t * word, ichar_t * ucword, int len,
@@ -86,15 +117,20 @@ static void	chk_suf P ((ichar_t * word, ichar_t * ucword, int len,
 static void	suf_list_chk P ((ichar_t * word, ichar_t * ucword, int len,
 		  struct flagptr * ind, int optflags, struct flagent * pfxent,
 		  int ignoreflagbits, int allhits));
-int		expand_pre P ((char * croot, ichar_t * rootword,
-		  MASKTYPE mask[], int option, char * extra));
-static int	pr_pre_expansion P ((char * croot, ichar_t * rootword,
-		  struct flagent * flent, MASKTYPE mask[], int option,
-		  char * extra));
-int		expand_suf P ((char * croot, ichar_t * rootword,
-		  MASKTYPE mask[], int optflags, int option, char * extra));
-static int	pr_suf_expansion P ((char * croot, ichar_t * rootword,
-		  struct flagent * flent, int option, char * extra));
+int		expand_pre P ((unsigned char * croot, ichar_t * rootword,
+		  MASKTYPE mask[], int option, unsigned char * extra));
+static int	gen_pre_expansion P ((unsigned char * croot,
+		  ichar_t * rootword, struct flagent * flent, MASKTYPE mask[],
+		  char exptable[MASKBITS][], int numexpansions));
+int		expand_suf P ((unsigned char * croot, ichar_t * rootword,
+		  MASKTYPE mask[], int optflags, int option,
+		  unsigned char * extra));
+int		expand_suf_into_table P ((unsigned char * croot,
+		  ichar_t * rootword, MASKTYPE mask[], int optflags,
+		  char exptable[MASKBITS][], int numexpansions));
+static int	gen_suf_expansion P ((unsigned char * croot,
+		  ichar_t * rootword, struct flagent * flent,
+		  char exptable[MASKBITS][], int numexpansions));
 static void	forcelc P ((ichar_t * dst, int len));
 
 /* Check possible affixes */
@@ -245,13 +281,9 @@ static void pfx_list_chk (word, ucword, len, optflags, sfxopts, ind,
 			}
 		    if (!allhits)
 			{
-#ifndef NO_CAPITALIZATION_SUPPORT
 			if (cap_ok (word, &hits[0], len))
 			    return;
 			numhits = 0;
-#else /* NO_CAPITALIZATION_SUPPORT */
-			return;
-#endif /* NO_CAPITALIZATION_SUPPORT */
 			}
 		    }
 		/*
@@ -443,13 +475,9 @@ static void suf_list_chk (word, ucword, len, ind, optflags, pfxent,
 			}
 		    if (!allhits)
 			{
-#ifndef NO_CAPITALIZATION_SUPPORT
 			if (cap_ok (word, &hits[0], len))
 			    return;
 			numhits = 0;
-#else /* NO_CAPITALIZATION_SUPPORT */
-			return;
-#endif /* NO_CAPITALIZATION_SUPPORT */
 			}
 		    }
 		}
@@ -461,42 +489,63 @@ static void suf_list_chk (word, ucword, len, ind, optflags, pfxent,
  * Expand a dictionary prefix entry
  */
 int expand_pre (croot, rootword, mask, option, extra)
-    char *			croot;		/* Char version of rootword */
+    unsigned char *		croot;		/* Char version of rootword */
     ichar_t *			rootword;	/* Root word to expand */
     register MASKTYPE		mask[];		/* Mask bits to expand on */
     int				option;		/* Option, see expandmode */
-    char *			extra;		/* Extra info to add to line */
+    unsigned char *		extra;		/* Extra info to add to line */
     {
     int				entcount;	/* No. of entries to process */
     int				explength;	/* Length of expansions */
+    char			exptable[MASKBITS][ICHARTOSSTR_SIZE];
+						/* Table of expansions */
     register struct flagent *
 				flent;		/* Current table entry */
+    int				numexpansions;	/* Size of expansion table */
 
-    for (flent = pflaglist, entcount = numpflags, explength = 0;
+    for (flent = pflaglist, entcount = numpflags, numexpansions = 0;
       entcount > 0;
       flent++, entcount--)
 	{
 	if (TSTMASKBIT (mask, flent->flagbit))
-	    explength +=
-	      pr_pre_expansion (croot, rootword, flent, mask, option, extra);
+	    numexpansions += gen_pre_expansion (croot, rootword, flent, mask,
+	      exptable, numexpansions);
+	}
+    for (explength = 0;  --numexpansions >= 0;  )
+	{
+	if (option == 3)
+	    (void) printf ("\n%s", (char *) croot);
+	if (option != 4)
+	    (void) printf (" %s%s", exptable[numexpansions], (char *) extra);
+	explength += strlen (exptable[numexpansions]);
 	}
     return explength;
     }
 
-/* Print a prefix expansion */
-static int pr_pre_expansion (croot, rootword, flent, mask, option, extra)
-    char *			croot;		/* Char version of rootword */
+/* Generate a prefix expansion */
+static int gen_pre_expansion (croot, rootword, flent, mask, exptable,
+  numexpansions)
+    unsigned char *		croot;		/* Char version of rootword */
     register ichar_t *		rootword;	/* Root word to expand */
     register struct flagent *	flent;		/* Current table entry */
     MASKTYPE			mask[];		/* Mask bits to expand on */
-    int				option;		/* Option, see	expandmode */
-    char *			extra;		/* Extra info to add to line */
+    char			exptable[MASKBITS][ICHARTOSSTR_SIZE];
+						/* Table of expansions */
+    int				numexpansions;	/* Size of expansion table */
     {
     int				cond;		/* Current condition number */
+    int				expno;		/* Expansion # being checked */
     register ichar_t *		nextc;		/* Next case choice */
+    int				numadded;	/* Number of expansios added */
+    register unsigned char *	result;		/* Result of expansion */
     int				tlen;		/* Length of tword */
     ichar_t			tword[INPUTWORDLEN + MAXAFFIXLEN]; /* Temp */
 
+    if (numexpansions == MASKBITS)
+	{
+	(void) fprintf (stderr, TGOOD_C_TOO_MANY_PRE);
+	return 0;
+	}
     tlen = icharlen (rootword);
     if (flent->numconds > tlen)
 	return 0;
@@ -563,34 +612,78 @@ static int pr_pre_expansion (croot, rootword, flent, mask, option, extra)
 	if (!myupper (*nextc))
 	    forcelc (tword, flent->affl);
 	}
-    if (option == 3)
-	(void) printf ("\n%s", croot);
-    if (option != 4)
-	(void) printf (" %s%s", ichartosstr (tword, 1), extra);
+    /*
+     * We've generated the result.  If it's not already in the
+     * expansion table, add it at the end.
+     */
+    result = ichartosstr (tword, 1);
+    numadded = 0;
+    for (expno = 0;  expno < numexpansions;  expno++)
+	{
+	if (strcmp (exptable[expno], (char *) result) == 0)
+	    break;
+	}
+    if (expno == numexpansions)
+	{
+	(void) strcpy (exptable[numexpansions], (char *) result);
+	numadded++;
+	numexpansions++;
+	}
     if (flent->flagflags & FF_CROSSPRODUCT)
-	return tlen
-	  + expand_suf (croot, tword, mask, FF_CROSSPRODUCT, option, extra);
-    else
-	return tlen;
+	numadded += expand_suf_into_table (croot, tword, mask, FF_CROSSPRODUCT,
+	  exptable, numexpansions);
+    return numadded;
     }
 
 /*
  * Expand a dictionary suffix entry
  */
 int expand_suf (croot, rootword, mask, optflags, option, extra)
-    char *			croot;		/* Char version of rootword */
+    unsigned char *		croot;		/* Char version of rootword */
     ichar_t *			rootword;	/* Root word to expand */
     register MASKTYPE		mask[];		/* Mask bits to expand on */
     int				optflags;	/* Affix option flags */
     int				option;		/* Option, see expandmode */
-    char *			extra;		/* Extra info to add to line */
+    unsigned char *		extra;		/* Extra info to add to line */
+    {
+    int				explength;	/* Length of expansions */
+    char			exptable[MASKBITS][ICHARTOSSTR_SIZE];
+						/* Table of expansions */
+    int				numexpansions;	/* Size of expansion table */
+
+    numexpansions =
+      expand_suf_into_table(croot, rootword, mask, optflags, exptable, 0);
+    for (explength = 0;  --numexpansions >= 0;  )
+	{
+	if (option == 3)
+	    (void) printf ("\n%s", (char *) croot);
+	if (option != 4)
+	    (void) printf (" %s%s", exptable[numexpansions], (char *) extra);
+	explength += strlen (exptable[numexpansions]);
+	}
+    return explength;
+    }
+
+/*
+ * Expand a dictionary suffix entry and insert it into an expansion table
+ */
+int expand_suf_into_table (croot, rootword, mask, optflags, exptable,
+  numexpansions)
+    unsigned char *		croot;		/* Char version of rootword */
+    ichar_t *			rootword;	/* Root word to expand */
+    register MASKTYPE		mask[];		/* Mask bits to expand on */
+    int				optflags;	/* Affix option flags */
+    char			exptable[MASKBITS][];
+						/* Table of expansions */
+    int				numexpansions;	/* Size of expansion table */
     {
     int				entcount;	/* No. of entries to process */
-    int				explength;	/* Length of expansions */
     register struct flagent *
 				flent;		/* Current table entry */
+    int				newexpansions;	/* New number. of expansions */
 
-    for (flent = sflaglist, entcount = numsflags, explength = 0;
+    newexpansions = numexpansions;
+    for (flent = sflaglist, entcount = numsflags;
       entcount > 0;
       flent++, entcount--)
 	{
@@ -598,57 +691,90 @@ int expand_suf (croot, rootword, mask, optflags, option, extra)
 	    {
 	    if ((optflags & FF_CROSSPRODUCT) == 0
 	      ||  (flent->flagflags & FF_CROSSPRODUCT))
-		explength +=
-		  pr_suf_expansion (croot, rootword, flent, option, extra);
+		newexpansions += gen_suf_expansion (croot, rootword, flent,
+		  exptable, newexpansions);
 	    }
 	}
-    return explength;
+    return newexpansions - numexpansions;
     }
 
-/* Print a suffix expansion */
-static int pr_suf_expansion (croot, rootword, flent, option, extra)
-    char *			croot;		/* Char version of rootword */
+/* Generate a suffix expansion */
+static int gen_suf_expansion (croot, rootword, flent, exptable, numexpansions)
+    unsigned char *		croot;		/* Char version of rootword */
     register ichar_t *		rootword;	/* Root word to expand */
     register struct flagent *	flent;		/* Current table entry */
-    int				option;		/* Option, see expandmode */
-    char *			extra;		/* Extra info to add to line */
+    char			exptable[MASKBITS][ICHARTOSSTR_SIZE];
+						/* Table of expansions */
+    int				numexpansions;	/* Size of expansion table */
     {
     int				cond;		/* Current condition number */
+    int				expno;		/* Expansion # being checked */
     register ichar_t *		nextc;		/* Next case choice */
+    register unsigned char *	result;		/* Result of expansion */
     int				tlen;		/* Length of tword */
     ichar_t			tword[INPUTWORDLEN + MAXAFFIXLEN]; /* Temp */
+    int				wascapitalized;	/* NZ if capitalized root */
 
+    if (numexpansions == MASKBITS)
+	{
+	(void) fprintf (stderr, TGOOD_C_TOO_MANY_SUF);
+	return 0;
+	}
     tlen = icharlen (rootword);
     cond = flent->numconds;
     if (cond > tlen)
 	return 0;
     if (tlen - flent->stripl <= 0)
 	return 0;
+    wascapitalized = myupper (rootword[0]);
     for (nextc = rootword + tlen;  --cond >= 0;  )
 	{
 	if ((flent->conds[mytoupper (*--nextc)] & (1 << cond)) == 0)
 	    return 0;
+	if (nextc > rootword  &&  myupper (*nextc))
+	    wascapitalized = 0;
+	}
+    while (--nextc > rootword)
+	{
+	if (myupper (*nextc))
+	    wascapitalized = 0;
 	}
     /*
      * The conditions are satisfied.  Copy the word, add the suffix,
      * and make it match the case of the last remaining character of the
      * root.  Again, this code carefully matches ins_cap and cap_ok.
+     *
+     * There is one tricky case to watch out for here.  If the root
+     * was simply capitalized, and the stripping removed all but the
+     * initial character of the root, the above algorithm will
+     * generate an all-caps result.  However, ins_cap and cap_ok would
+     * generate a simple capitalization, because a captype of
+     * CAPITALIZED would override FOLLOWCASE and ALLCAPS.  So we have
+     * to deal with this as a special case.
      */
     (void) icharcpy (tword, rootword);
     nextc = tword + tlen - flent->stripl;
     if (flent->affl)
 	{
 	(void) icharcpy (nextc, flent->affix);
-	if (!myupper (nextc[-1]))
+	if ((nextc == tword + 1  &&  wascapitalized)
+	  ||  !myupper (nextc[-1]))
 	    forcelc (nextc, flent->affl);
 	}
     else
 	*nextc = 0;
-    if (option == 3)
-	(void) printf ("\n%s", croot);
-    if (option != 4)
-	(void) printf (" %s%s", ichartosstr (tword, 1), extra);
-    return tlen + flent->affl - flent->stripl;
+    /*
+     * We've generated the result.  If it's not already in the
+     * expansion table, add it at the end.
+     */
+    result = ichartosstr (tword, 1);
+    for (expno = 0;  expno < numexpansions;  expno++)
+	{
+	if (strcmp (exptable[expno], (char *) result) == 0)
+	    return 0;
+	}
+    (void) strcpy (exptable[numexpansions], (char *) result);
+    return 1;
     }
 
 static void forcelc (dst, len)			/* Force to lowercase */
