@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: tree.c,v 1.55 1994/10/25 05:46:27 geoff Exp $";
+    "$Id: tree.c,v 1.61 2000/08/22 10:52:25 geoff Exp $";
 #endif
 
 /*
@@ -9,7 +9,7 @@ static char Rcs_Id[] =
  * Pace Willisson, 1983
  * Hash support added by Geoff Kuenning, 1987
  *
- * Copyright 1987, 1988, 1989, 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * Copyright 1987-1989, 1992, 1993, 1999, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,6 +48,27 @@ static char Rcs_Id[] =
 
 /*
  * $Log: tree.c,v $
+ * Revision 1.61  2000/08/22 10:52:25  geoff
+ * Fix some compiler warnings.
+ *
+ * Revision 1.60  2000/06/26 20:43:50  geoff
+ * Fix a minor lint bug.  If HOME is unset, still attempt to find the
+ * personal dictionary using all ways that don't depend on HOME (in
+ * particular, the -p switch).
+ *
+ * Revision 1.59  1999/01/18 03:28:45  geoff
+ * Turn many char declarations into unsigned char, so that we won't have
+ * sign-extension problems.
+ *
+ * Revision 1.58  1999/01/07  01:57:46  geoff
+ * Update the copyright.
+ *
+ * Revision 1.57  1997/12/02  06:25:11  geoff
+ * Get rid of some compile options that really shouldn't be optional.
+ *
+ * Revision 1.56  1995/01/08  23:23:49  geoff
+ * Support PDICTHOME for DOS purposes.
+ *
  * Revision 1.55  1994/10/25  05:46:27  geoff
  * Fix a comment that looked to some compilers like it might be nested.
  *
@@ -67,7 +88,7 @@ void		treeinit P ((char * p, char * LibDict));
 static FILE *	trydict P ((char * dictname, char * home,
 		  char * prefix, char * suffix));
 static void	treeload P ((FILE * dictf));
-void		treeinsert P ((char * word, int wordlen, int keep));
+void		treeinsert P ((unsigned char * word, int wordlen, int keep));
 static struct dent * tinsert P ((struct dent * proto));
 struct dent *	treelookup P ((ichar_t * word));
 #if SORTPERSONAL != 0
@@ -98,7 +119,7 @@ static int goodsizes[] =
 
 static char		personaldict[MAXPATHLEN];
 static FILE *		dictf;
-static			newwords = 0;
+static int		newwords = 0;
 
 void treeinit (p, LibDict)
     char *		p;		/* Value specified in -p switch */
@@ -119,11 +140,17 @@ void treeinit (p, LibDict)
     if (p == NULL)
 	p = getenv (PDICTVAR);
     /*
-    ** if p exists and begins with '/' we don't really need HOME,
-    ** but it's not very likely that HOME isn't set anyway.
+    ** Figure out the user's home.  If HOME is unset (which can happen
+    ** if ispell is invoked unusually, such as from a daemon), we
+    ** won't look for any HOME personal dictionaries.  Exception:  If
+    ** PDICTHOME is set, we use it as a default for HOME.
     */
-    if ((h = getenv ("HOME")) == NULL)
-	return;
+    if ((h = getenv (HOME)) == NULL)
+	{
+#ifdef PDICTHOME
+	h = PDICTHOME;
+#endif /* PDICTHOME */
+	}
 
     if (p == NULL)
 	{
@@ -147,21 +174,27 @@ void treeinit (p, LibDict)
 	 *     file.
 	 */
 	dictf = trydict (personaldict, (char *) NULL, DEFPDICT, LibDict);
-	secondf = trydict (seconddict, h, DEFPDICT, LibDict);
+	if (h == NULL)
+	    secondf = NULL;
+	else
+	    secondf = trydict (seconddict, h, DEFPDICT, LibDict);
 	if (dictf == NULL  &&  secondf == NULL)
 	    {
 	    dictf = trydict (personaldict, (char *) NULL, DEFPDICT, DEFPAFF);
-	    secondf = trydict (seconddict, h, DEFPDICT, DEFPAFF);
+	    if (h != NULL)
+		secondf = trydict (seconddict, h, DEFPDICT, DEFPAFF);
 	    }
 	if (dictf == NULL  &&  secondf == NULL)
 	    {
 	    dictf = trydict (personaldict, (char *) NULL, OLDPDICT, LibDict);
-	    secondf = trydict (seconddict, h, OLDPDICT, LibDict);
+	    if (h != NULL)
+		secondf = trydict (seconddict, h, OLDPDICT, LibDict);
 	    }
 	if (dictf == NULL  &&  secondf == NULL)
 	    {
 	    dictf = trydict (personaldict, (char *) NULL, OLDPDICT, OLDPAFF);
-	    secondf = trydict (seconddict, h, OLDPDICT, OLDPAFF);
+	    if (h != NULL)
+		secondf = trydict (seconddict, h, OLDPDICT, OLDPAFF);
 	    }
 	if (personaldict[0] == '\0')
 	    {
@@ -205,7 +238,9 @@ void treeinit (p, LibDict)
 	    ** The user gave us a relative pathname.  We will try it
 	    ** locally, and if that doesn't work, we'll try the home
 	    ** directory.  If neither exists, it will be created in
-	    ** the home directory if words are added.
+	    ** the home directory if words are added, unless HOME was
+	    ** unset, in which case the relative pathname will be used
+	    ** as a creation point.
 	    */
 	    (void) strcpy (personaldict, p);
 	    if ((dictf = fopen (personaldict, "r")) != NULL)
@@ -213,7 +248,7 @@ void treeinit (p, LibDict)
 		treeload (dictf);
 		(void) fclose (dictf);
 		}
-	    else if (!abspath)
+	    else if (!abspath  &&  h != NULL)
 		{
 		/* Try the home */
 		(void) sprintf (personaldict, "%s/%s", h, p);
@@ -273,28 +308,24 @@ static void treeload (loadfile)
     char		buf[BUFSIZ];	/* Buffer for reading pers dict */
 
     while (fgets (buf, sizeof buf, loadfile) != NULL)
-	treeinsert (buf, sizeof buf, 1);
+	treeinsert ((unsigned char *) buf, sizeof buf, 1);
     newwords = 0;
     }
 
 void treeinsert (word, wordlen, keep)
-    char *		word;	/* Word to insert - must be canonical */
+    unsigned char *	word;	/* Word to insert - must be canonical */
     int			wordlen; /* Length of the word buffer */
     int			keep;
     {
-    register int	i;
+    register unsigned int i;
     struct dent		wordent;
     register struct dent * dp;
     struct dent *	olddp;
-#ifndef NO_CAPITALIZATION_SUPPORT
     struct dent *	newdp;
-#endif
     struct dent *	oldhtab;
-    int			oldhsize;
+    unsigned int	oldhsize;
     ichar_t		nword[INPUTWORDLEN + MAXAFFIXLEN];
-#ifndef NO_CAPITALIZATION_SUPPORT
     int			isvariant;
-#endif
 
     /*
      * Expand hash table when it is MAXPCT % full.
@@ -342,22 +373,9 @@ void treeinsert (word, wordlen, keep)
 		dp = &oldhtab[i];
 		if (dp->flagfield & USED)
 		    {
-#ifdef NO_CAPITALIZATION_SUPPORT
-		    (void) tinsert (dp);
-#else
 		    newdp = tinsert (dp);
 		    isvariant = (dp->flagfield & MOREVARIANTS);
-#endif
 		    dp = dp->next;
-#ifdef NO_CAPITALIZATION_SUPPORT
-		    while (dp != NULL)
-			{
-			(void) tinsert (dp);
-			olddp = dp;
-			dp = dp->next;
-			free ((char *) olddp);
-			}
-#else
 		    while (dp != NULL)
 			{
 			if (isvariant)
@@ -378,7 +396,6 @@ void treeinsert (word, wordlen, keep)
 			    free ((char *) olddp);
 			    }
 			}
-#endif
 		    }
 		}
 	    if (oldhtab != NULL)
@@ -413,10 +430,8 @@ void treeinsert (word, wordlen, keep)
 	{
 	/* It's new. Insert the word. */
 	dp = tinsert (&wordent);
-#ifndef NO_CAPITALIZATION_SUPPORT
 	if (captype (dp->flagfield) == FOLLOWCASE)
 	   (void) addvheader (dp);
-#endif
 	}
     newwords |= keep;
     }
@@ -430,10 +445,7 @@ static struct dent * tinsert (proto)
     register struct dent * php;		/* Prev. value of hp, for chaining */
 
     if (strtoichar (iword, proto->word, sizeof iword, 1))
-	(void) fprintf (stderr, WORD_TOO_LONG (proto->word));
-#ifdef NO_CAPITALIZATION_SUPPORT
-    upcase (iword);
-#endif
+	(void) fprintf (stderr, WORD_TOO_LONG ((char *) proto->word));
     hcode = hash (iword, pershsize);
     php = NULL;
     hp = &pershtab[hcode];
@@ -467,17 +479,15 @@ struct dent * treelookup (word)
 
     if (pershsize <= 0)
 	return NULL;
-    (void) ichartostr (chword, word, sizeof chword, 1);
+    (void) ichartostr ((unsigned char *) chword, word, sizeof chword, 1);
     hcode = hash (word, pershsize);
     hp = &pershtab[hcode];
     while (hp != NULL  &&  (hp->flagfield & USED))
 	{
-	if (strcmp (chword, hp->word) == 0)
+	if (strcmp (chword, (char *) hp->word) == 0)
 	    break;
-#ifndef NO_CAPITALIZATION_SUPPORT
 	while (hp->flagfield & MOREVARIANTS)
 	    hp = hp->next;
-#endif
 	hp = hp->next;
 	}
     if (hp != NULL  &&  (hp->flagfield & USED))
@@ -540,10 +550,8 @@ void treeoutput ()
 		{
 		if ((lent->flagfield & (USED | KEEP)) == (USED | KEEP))
 		    pdictsize++;
-#ifndef NO_CAPITALIZATION_SUPPORT
 		while (lent->flagfield & MOREVARIANTS)
 		  lent = lent->next;
-#endif
 		}
 	    }
 	for (cent = hashtbl, ehtab = hashtbl + hashsize;
@@ -559,10 +567,8 @@ void treeoutput ()
 		** test below.  This test will appear
 		** several more times in this routine.
 		*/
-#ifndef NO_CAPITALIZATION_SUPPORT
 		if (captype (cent->flagfield) != FOLLOWCASE
 		  &&  cent->word != NULL)
-#endif
 		    pdictsize++;
 		}
 	    }
@@ -580,10 +586,8 @@ void treeoutput ()
 		if ((lent->flagfield & (USED | KEEP)) == (USED | KEEP))
 		    {
 		    toutent (dictf, lent, 1);
-#ifndef NO_CAPITALIZATION_SUPPORT
 		    while (lent->flagfield & MOREVARIANTS)
 			lent = lent->next;
-#endif
 		    }
 		}
 	    }
@@ -593,10 +597,8 @@ void treeoutput ()
 	    {
 	    if ((cent->flagfield & (USED | KEEP)) == (USED | KEEP))
 		{
-#ifndef NO_CAPITALIZATION_SUPPORT
 		if (captype (cent->flagfield) != FOLLOWCASE
 		  &&  cent->word != NULL)
-#endif
 		    toutent (dictf, cent, 1);
 		}
 	    }
@@ -618,10 +620,8 @@ void treeoutput ()
 	    if ((lent->flagfield & (USED | KEEP)) == (USED | KEEP))
 		{
 		*sortptr++ = lent;
-#ifndef NO_CAPITALIZATION_SUPPORT
 		while (lent->flagfield & MOREVARIANTS)
 		    lent = lent->next;
-#endif
 		}
 	    }
 	}
@@ -629,10 +629,8 @@ void treeoutput ()
 	{
 	if ((cent->flagfield & (USED | KEEP)) == (USED | KEEP))
 	    {
-#ifndef NO_CAPITALIZATION_SUPPORT
 	    if (captype (cent->flagfield) != FOLLOWCASE
 	      &&  cent->word != NULL)
-#endif
 		*sortptr++ = cent;
 	    }
 	}
@@ -661,8 +659,8 @@ VOID * mymalloc (size)
 void myfree (ptr)
     VOID * ptr;
     {
-    if (hashstrings != NULL  &&  (char *) ptr >= hashstrings
-      &&  (char *) ptr <= hashstrings + hashheader.stringsize)
+    if (hashstrings != NULL  &&  (unsigned char *) ptr >= hashstrings
+      &&  (unsigned char *) ptr <= hashstrings + hashheader.stringsize)
 	return;			/* Can't free stuff in hashstrings */
     free (ptr);
     }
@@ -700,11 +698,11 @@ char * do_regex_lookup (expr, whence)
     for (  ; curindex < hashsize;  curent++, curindex++)
 	{
         if (curent->word != NULL
-          &&  REGEX (cmp_expr, curent->word, dummy) != NULL)
+          &&  REGEX (cmp_expr, (char *) curent->word, dummy) != NULL)
 	    {
 	    curindex++;
 	    /* Everybody's gotta write a wierd expression once in a while! */
-	    return curent++->word;
+	    return (char *) curent++->word;
 	    }
 	}
     /* Try the personal dictionary too */
@@ -712,7 +710,7 @@ char * do_regex_lookup (expr, whence)
 	{
         if ((curpersent->flagfield & USED) != 0
           &&  curpersent->word != NULL
-          &&  REGEX (cmp_expr, curpersent->word, dummy) != NULL)
+          &&  REGEX (cmp_expr, (char *) curpersent->word, dummy) != NULL)
 	    {
 	    curpersindex++;
 	    /* Everybody's gotta write a wierd expression once in a while! */
