@@ -8,10 +8,13 @@
  */
 
 static char * Version_ID[] = {
-    "@(#) International Ispell Version 3.1.20 10/10/95",
+    "@(#) International Ispell Version 3.2.03 06/10/01",
     "@(#) Copyright (c), 1983, by Pace Willisson",
-    "@(#) International version Copyright (c) 1987, 1988, 1990-1995,",
-    "@(#) by Geoff Kuenning, Granada Hills, CA.  All rights reserved.",
+    "@(#) International version Copyright (c) 1987, 1988, 1990-1995, 1999,",
+    "@(#) 2001, by Geoff Kuenning, Claremont, CA.  All rights reserved.",
+    "@(#)",
+    "@(#) Report bugs to ispell-bugs@itcorp.com.  Report Emacs-related bugs",
+    "@(#) to ispell-el-bugs@itcorp.com.",
     "@(#)",
     "@(#) Redistribution and use in source and binary forms, with or without",
     "@(#) modification, are permitted provided that the following conditions",
@@ -51,10 +54,22 @@ static char * Version_ID[] = {
 };
 
 static char RCS_Version_ID[] =
-    "$Id: version.h,v 1.44 1995/10/11 05:03:28 geoff Exp $";
+    "$Id: version.h,v 1.48 2001/06/10 23:54:57 geoff Exp $";
 
 /*
  * $Log: version.h,v $
+ * Revision 1.48  2001/06/10 23:54:57  geoff
+ * Bump verson to reflect latest bug fix.
+ *
+ * Revision 1.47  2001/06/06 23:50:26  geoff
+ * Bump to 3.2.02 in preparation for release.
+ *
+ * Revision 1.46  1999/01/18 02:15:44  geoff
+ * Bump to next version
+ *
+ * Revision 1.45  1999/01/07  01:23:00  geoff
+ * Update the copyright and the version number preparatory to release.
+ *
  * Revision 1.44  1995/10/11  05:03:28  geoff
  * Upgrade to patch level 20
  *
