@@ -1,10 +1,10 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: xgets.c,v 1.22 1994/09/16 04:48:34 geoff Exp $";
+    "$Id: xgets.c,v 1.24 1999/01/18 03:28:47 geoff Exp $";
 #endif
 
 /*
- * Copyright 1987, 1988, 1989, 1992, 1993, Geoff Kuenning, Granada Hills, CA
+ * Copyright 1987, 1988, 1989, 1992, 1993, 1999, Geoff Kuenning, Claremont, CA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,6 +43,12 @@ static char Rcs_Id[] =
 
 /*
  * $Log: xgets.c,v $
+ * Revision 1.24  1999/01/18 03:28:47  geoff
+ * Get of a warning message caused by poor style.
+ *
+ * Revision 1.23  1999/01/07  01:23:01  geoff
+ * Update the copyright.
+ *
  * Revision 1.22  1994/09/16  04:48:34  geoff
  * Be sure to deliver newlines to the caller, so that it can tell whether
  * or not a complete line was read.
@@ -134,7 +140,7 @@ char * xgets (str, size, stream)
 		{
 		FILE *	f;
 
-		if (f = fopen (file_name, "r"))
+		if ((f = fopen (file_name, "r")) != NULL)
 		    *(++current_F) = f;
 		}
 	    s = str;
